Package: datefixR
Title: Standardize Dates in Different Formats or with Missing Data
Version: 2.0.1
Maintainer: Nathan Constantine-Cooke <nathan.constantine-cooke@ed.ac.uk>
Authors@R: c(
    person("Nathan", "Constantine-Cooke", , "nathan.constantine-cooke@ed.ac.uk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4437-8713")),
    person("Jonathan", "Kitt", , "jonathan.kitt@protonmail.com", role = c("ctb", "trl")),
    person("Antonio J.", "Pérez-Luque", , "ajpelu@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0002-1747-0469")),
    person("Daniel", "Possenriede", , "possenriede+r@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0002-6738-9845")),
    person("Michal", "Lauer", , "michal.lauer.25@gmail.com", role = c("ctb", "trl")),
    person("Kaique dos S.", "Alves", , "kaiquedsalves@gmail.com", role = "rev",
           comment = c(ORCID = "0000-0001-9187-0252")),
    person("Al-Ahmadgaid B.", "Asaad", , "alahmadgaid@gmail.com", role = "rev",
           comment = c(ORCID = "0000-0003-3784-8593")),
    person("Anatoly", "Tsyplenkov", , "atsyplenkov@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0003-4144-8402")),
    person("Chitra M.", "Saraswati", , "chitra.m.saraswati@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0002-8159-0414"))
  )
Description: There are many different formats dates are commonly
    represented with: the order of day, month, or year can differ,
    different separators ("-", "/", or whitespace) can be used, months can
    be numerical, names, or abbreviations and year given as two digits or
    four. 'datefixR' takes dates in all these different formats and
    converts them to R's built-in date class. If 'datefixR' cannot
    standardize a date, such as because it is too malformed, then the user
    is told which date cannot be standardized and the corresponding ID for
    the row. 'datefixR' also allows the imputation of missing days and
    months with user-controlled behavior.
License: GPL (>= 3)
URL: https://docs.ropensci.org/datefixR/,
        https://github.com/ropensci/datefixR
BugReports: https://github.com/ropensci/datefixR/issues
Depends: R (>= 4.2)
Imports: lifecycle, rlang
Suggests: anytime, DT, future, future.apply, htmltools, knitr,
        parsedate, pkgbuild, png, readr, readxl, rmarkdown, shiny,
        shinytest2, spelling, testthat (>= 3.0.0), withr
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.3
Config/rextendr/version: 0.5.0
SystemRequirements: Cargo (Rust's package manager), rustc, xz
NeedsCompilation: yes
Packaged: 2026-04-27 13:44:59 UTC; nconsta2
Author: Nathan Constantine-Cooke [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4437-8713>),
  Jonathan Kitt [ctb, trl],
  Antonio J. Pérez-Luque [ctb, trl] (ORCID:
    <https://orcid.org/0000-0002-1747-0469>),
  Daniel Possenriede [ctb, trl] (ORCID:
    <https://orcid.org/0000-0002-6738-9845>),
  Michal Lauer [ctb, trl],
  Kaique dos S. Alves [rev] (ORCID:
    <https://orcid.org/0000-0001-9187-0252>),
  Al-Ahmadgaid B. Asaad [rev] (ORCID:
    <https://orcid.org/0000-0003-3784-8593>),
  Anatoly Tsyplenkov [ctb, trl] (ORCID:
    <https://orcid.org/0000-0003-4144-8402>),
  Chitra M. Saraswati [ctb, trl] (ORCID:
    <https://orcid.org/0000-0002-8159-0414>)
Repository: CRAN
Date/Publication: 2026-04-27 14:00:09 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-05-01 17:29:58 UTC; unix
Archs: datefixR.so.dSYM

Package: datefixR
Title: Standardize Dates in Different Formats or with Missing Data
Version: 2.0.1
Maintainer: Nathan Constantine-Cooke <nathan.constantine-cooke@ed.ac.uk>
Authors@R: c(
    person("Nathan", "Constantine-Cooke", , "nathan.constantine-cooke@ed.ac.uk", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4437-8713")),
    person("Jonathan", "Kitt", , "jonathan.kitt@protonmail.com", role = c("ctb", "trl")),
    person("Antonio J.", "Pérez-Luque", , "ajpelu@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0002-1747-0469")),
    person("Daniel", "Possenriede", , "possenriede+r@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0002-6738-9845")),
    person("Michal", "Lauer", , "michal.lauer.25@gmail.com", role = c("ctb", "trl")),
    person("Kaique dos S.", "Alves", , "kaiquedsalves@gmail.com", role = "rev",
           comment = c(ORCID = "0000-0001-9187-0252")),
    person("Al-Ahmadgaid B.", "Asaad", , "alahmadgaid@gmail.com", role = "rev",
           comment = c(ORCID = "0000-0003-3784-8593")),
    person("Anatoly", "Tsyplenkov", , "atsyplenkov@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0003-4144-8402")),
    person("Chitra M.", "Saraswati", , "chitra.m.saraswati@gmail.com", role = c("ctb", "trl"),
           comment = c(ORCID = "0000-0002-8159-0414"))
  )
Description: There are many different formats dates are commonly
    represented with: the order of day, month, or year can differ,
    different separators ("-", "/", or whitespace) can be used, months can
    be numerical, names, or abbreviations and year given as two digits or
    four. 'datefixR' takes dates in all these different formats and
    converts them to R's built-in date class. If 'datefixR' cannot
    standardize a date, such as because it is too malformed, then the user
    is told which date cannot be standardized and the corresponding ID for
    the row. 'datefixR' also allows the imputation of missing days and
    months with user-controlled behavior.
License: GPL (>= 3)
URL: https://docs.ropensci.org/datefixR/,
        https://github.com/ropensci/datefixR
BugReports: https://github.com/ropensci/datefixR/issues
Depends: R (>= 4.2)
Imports: lifecycle, rlang
Suggests: anytime, DT, future, future.apply, htmltools, knitr,
        parsedate, pkgbuild, png, readr, readxl, rmarkdown, shiny,
        shinytest2, spelling, testthat (>= 3.0.0), withr
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
Language: en-US
LazyData: true
RoxygenNote: 7.3.3
Config/rextendr/version: 0.5.0
SystemRequirements: Cargo (Rust's package manager), rustc, xz
NeedsCompilation: yes
Packaged: 2026-04-27 13:44:59 UTC; nconsta2
Author: Nathan Constantine-Cooke [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4437-8713>),
  Jonathan Kitt [ctb, trl],
  Antonio J. Pérez-Luque [ctb, trl] (ORCID:
    <https://orcid.org/0000-0002-1747-0469>),
  Daniel Possenriede [ctb, trl] (ORCID:
    <https://orcid.org/0000-0002-6738-9845>),
  Michal Lauer [ctb, trl],
  Kaique dos S. Alves [rev] (ORCID:
    <https://orcid.org/0000-0001-9187-0252>),
  Al-Ahmadgaid B. Asaad [rev] (ORCID:
    <https://orcid.org/0000-0003-3784-8593>),
  Anatoly Tsyplenkov [ctb, trl] (ORCID:
    <https://orcid.org/0000-0003-4144-8402>),
  Chitra M. Saraswati [ctb, trl] (ORCID:
    <https://orcid.org/0000-0002-8159-0414>)
Repository: CRAN
Date/Publication: 2026-04-27 14:00:09 UTC

# Generated by roxygen2: do not edit by hand

export(fix_date)
export(fix_date_app)
export(fix_date_char)
export(fix_date_df)
export(fix_dates)
importFrom(Rcpp,sourceCpp)
importFrom(lifecycle,deprecated)
useDynLib(datefixR, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# datefixR <img src="man/figures/logo.png" align="right" width="150"/>

<!-- badges: start -->

| Usage                                                                                                                                                                    | Release                                                                                                                                          | Development                                                                                                                                                                                            | Translation Status                                                                                                                  |
| ------------------------------------------------------------------------------------------------------------------------------------------------------------------------ | ------------------------------------------------------------------------------------------------------------------------------------------------ | ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------ | ----------------------------------------------------------------------------------------------------------------------------------- |
| ![R](https://img.shields.io/badge/r-%23276DC3.svg?style=for-the-badge&logo=r&logoColor=white)                                                                            | [![CRAN\_Status\_Badge](https://www.r-pkg.org/badges/version/datefixR)](https://cran.r-project.org/package=datefixR)                             | ![Rust language](https://img.shields.io/badge/Rust-D34516?style=for-the-badge&logo=rust&logoColor=#E57324)                                                                                             | [![German localization](https://gitlocalize.com/repo/8364/de/badge.svg)](https://gitlocalize.com/repo/8364/de?utm_source=badge)     |
| [![License: GPL-3](https://img.shields.io/badge/License-GPL3-green.svg)](https://opensource.org/license/gpl-3-0)                                                         | [![datefixR status badge](https://ropensci.r-universe.dev/badges/datefixR)](https://ropensci.r-universe.dev/datefixR)                            | [![R build status](https://github.com/ropensci/datefixR/workflows/CI/badge.svg)](https://github.com/ropensci/datefixR/actions)                                                                         | [![Spanish localization](https://gitlocalize.com/repo/8364/es/badge.svg)](https://gitlocalize.com/repo/8364/es?utm_source=badge)    |
| [![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/grand-total/datefixR?color=blue)](https://r-pkg.org/pkg/datefixR)                                    | [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.5655311.svg)](https://doi.org/10.5281/zenodo.5655311)                                        | [![Project Status: Active – The project has reached a stable, usable state and is being actively developed.](https://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active) | [![French localization](https://gitlocalize.com/repo/8364/fr/badge.svg)](https://gitlocalize.com/repo/8364/fr?utm_source=badge)     |
| ![website status](https://img.shields.io/website?down_color=red&down_message=offline&up_color=green&up_message=online&url=https%3A%2F%2Fdocs.ropensci.org%2FdatefixR%2F) | [![Status at rOpenSci Software Peer Review](https://badges.ropensci.org/533_status.svg)](https://github.com/ropensci/software-review/issues/533) | [![codecov](https://codecov.io/gh/ropensci/datefixR/branch/main/graph/badge.svg?token=zycOVwlq1m)](https://app.codecov.io/gh/ropensci/datefixR)                                                        | [![Indonesian localization](https://gitlocalize.com/repo/8364/id/badge.svg)](https://gitlocalize.com/repo/8364/id?utm_source=badge) |
|                                                                                                                                                                          |                                                                                                                                                  | [![Tidyverse style guide](https://img.shields.io/static/v1?label=Code%20Style&message=Tidyverse&color=1f1c30)](https://style.tidyverse.org)                                                            | [![Russian localization](https://gitlocalize.com/repo/8364/ru/badge.svg)](https://gitlocalize.com/repo/8364/ru?utm_source=badge)    |

<!-- badges: end -->

**`datefixR` is an R package that automatically standardizes messy date
data into consistent, machine-readable formats.** Whether you’re dealing
with free-text web form entries like “02 05 92”, “2020-may-01”, or “le 3
mars 2013”, `datefixR` intelligently parses diverse date formats and
converts them to R’s standard Date class. Under the hood, `datefixR`
uses Rust for fast and memory-safe parsing.

**Key features:**

  - **Smart parsing**: Handles mixed date formats, separators, and
    representations in a single dataset.
  - **Multilingual support**: Recognizes dates in English, French,
    German, Spanish, Indonesian, and Russian.
  - **Missing data imputation**: User-controlled behavior for incomplete
    dates (missing days/months).
  - **Error reporting**: If a date cannot be parsed, the user is
    informed of the provided date and associated row ID, allowing for
    easier debugging and correction.
  - **Excel compatibility**: Supports both R and Excel numeric date
    representations.
  - **Shiny integration**: Interactive web app for data exploration and
    cleaning. <br>

<img src="man/figures/example.svg" width="800"/>

## Quick Start

Here’s a simple example showing how `datefixR` cleans messy date data:

``` r
library(datefixR)

# Create some messy date data
messy_dates <- c("02/05/92", "2020-may-01", "le 3 mars 2013", "1996")
messy_df <- data.frame(id = 1:4, dates = messy_dates)
print(messy_df)
#>   id          dates
#> 1  1       02/05/92
#> 2  2    2020-may-01
#> 3  3 le 3 mars 2013
#> 4  4           1996

# Clean the dates
clean_dates <- fix_date_char(messy_dates) # Clean a character vector
clean_df <- fix_date_df(messy_df, "dates") # Clean a column of a dataframe
print(clean_df)
#>   id      dates
#> 1  1 1992-05-02
#> 2  2 2020-05-01
#> 3  3 2013-03-03
#> 4  4 1996-07-01
```

The package automatically standardizes dates in different formats (named
months, various separators, incomplete dates) into R’s standard
`yyyy-mm-dd` format. When dates are partially missing (like the day or
month), they can be imputed. Whilst imputation defaults to July 1st for
incomplete dates, this behaviour can be changed including the prevention
of any imputation.

## Installation

### Stable Release (Recommended)

`datefixR` is available on CRAN:

``` r
install.packages("datefixR")
```

### Latest Stable (r-universe)

For the most up-to-date stable version via
[r-universe](https://r-universe.dev/search):

``` r
# Enable universe(s) by ropensci
options(repos = c(
  ropensci = "https://ropensci.r-universe.dev",
  CRAN = "https://cloud.r-project.org"
))

install.packages("datefixR")
```

## Getting Started

## Package vignette

`datefixR` has a “Getting Started” vignette which describes how to use
this package in more detail than this page. View the vignette by either
calling

``` r
browseVignettes("datefixR")
```

or visiting the vignette on the [package
website](https://docs.ropensci.org/datefixR/articles/datefixR.html)

Additional vignettes are available describing `datefixR`’s localization
features and how to use the Shiny app.

## Usage

`datefixR` provides flexible date standardization capabilities across
different data structures and formats. This section demonstrates various
use cases with practical examples.

### Character Vector Cleaning

The most basic use case involves cleaning a character vector of messy
dates using `fix_date_char()`:

``` r
library(datefixR)

# Mixed format dates
messy_dates <- c(
  "02/05/92", # US format, 2-digit year
  "2020-may-01", # ISO with named month
  "le 3 mars 2013", # French format
  "1996", # Year only
  "22.07.1977", # European format
  "jan 2020" # Month-year only
)

# Clean all dates at once
clean_dates <- fix_date_char(messy_dates)
print(clean_dates)
#> [1] "1992-05-02" "2020-05-01" "2013-03-03" "1996-07-01" "1977-07-22"
#> [6] "2020-01-01"
```

This function automatically handles various separators (“-”, “/”, “.”,
spaces), different date orders, named months in multiple languages, and
incomplete dates.

### Data Frame Cleaning

For structured data, use `fix_date_df()` to clean multiple date columns
simultaneously:

``` r
# Load example dataset
data("exampledates")
knitr::kable(exampledates)
```

| id | some.dates              | some.more.dates |
| :- | :---------------------- | :-------------- |
| 1  | 02 05 92                | 2015            |
| 2  | 01-04-2020              | 02/05/00        |
| 3  | 1996/05/01              | 05/1990         |
| 4  | 2020-may-01             | 2012-08         |
| 5  | 02-04-96                | jan 2020        |
| 6  | le 3 mars 2013          | 22.07.1977      |
| 7  | 7 de septiembre de 2014 | 13821           |

``` r

# Fix multiple columns
fixed_df <- fix_date_df(exampledates, c("some.dates", "some.more.dates"))
knitr::kable(fixed_df)
```

| id | some.dates | some.more.dates |
| :- | :--------- | :-------------- |
| 1  | 1992-05-02 | 2015-07-01      |
| 2  | 2020-04-01 | 2000-05-02      |
| 3  | 1996-05-01 | 1990-05-01      |
| 4  | 2020-05-01 | 2012-08-01      |
| 5  | 1996-04-02 | 2020-01-01      |
| 6  | 2013-03-03 | 1977-07-22      |
| 7  | 2014-09-07 | 2007-11-04      |

The function preserves non-date columns and provides detailed error
reporting if any dates fail to parse.

### Excel Serial Numbers

`datefixR` supports both R and Excel numeric date representations:

``` r
# R serial dates (days since 1970-01-01)
r_serial <- "19539" # Represents 2023-07-01
fix_date_char(r_serial)
#> [1] "2023-07-01"

# Excel serial dates (days since 1900-01-01, accounting for Excel's leap year bug)
excel_serial <- "45108" # Also represents 2023-07-01
fix_date_char(excel_serial, excel = TRUE)
#> [1] "2023-07-01"

# Mixed serial and text dates
mixed_dates <- c("45108", "2023-07-01", "july 1 2023")
fix_date_char(mixed_dates, excel = TRUE)
#> [1] "2023-07-01" "2023-07-01" "2023-07-01"
```

This is particularly useful when importing data from Excel spreadsheets
where dates may have been converted to serial numbers.

### MDY vs DMY Detection

By default, `datefixR` assumes day-first (DMY) over month-first format
when the date order is ambiguous. However, you can specify month-first
(MDY) format:

``` r
# Ambiguous dates that could be interpreted as either MDY or DMY
ambiguous_dates <- c("01/02/2023", "03/04/2023", "05/06/2023")

# Default: Day-first (DMY) interpretation
dmy_result <- fix_date_char(ambiguous_dates)
print(dmy_result)
#> [1] "2023-02-01" "2023-04-03" "2023-06-05"

# Month-first (MDY) interpretation
mdy_result <- fix_date_char(ambiguous_dates, format = "mdy")
print(mdy_result)
#> [1] "2023-01-02" "2023-03-04" "2023-05-06"
```

### Missing Day/Month Imputation

`datefixR` provides flexible control over how missing date components
are imputed:

``` r
# Incomplete dates requiring imputation
incomplete_dates <- c("2023", "05/2023", "2023-08", "march 2022")

# Default imputation: missing month = July (07), missing day = 1st
default_impute <- fix_date_char(incomplete_dates)
print(default_impute)
#> [1] "2023-07-01" "2023-05-01" "2023-08-01" "2022-03-01"

# Custom imputation: missing month = January (01), missing day = 15th
custom_impute <- fix_date_char(incomplete_dates,
  month.impute = 1,
  day.impute = 15
)
print(custom_impute)
#> [1] "2023-01-15" "2023-05-15" "2023-08-15" "2022-03-15"

# For data frames, apply the same logic
incomplete_df <- data.frame(
  id = 1:4,
  dates = incomplete_dates
)

fixed_incomplete <- fix_date_df(incomplete_df, "dates",
  month.impute = 12, # December
  day.impute = 31
) # Last day
knitr::kable(fixed_incomplete)
```

| id | dates      |
| -: | :--------- |
|  1 | 2023-12-31 |
|  2 | 2023-05-31 |
|  3 | 2023-08-31 |
|  4 | 2022-03-31 |

This flexibility allows you to choose imputation strategies that make
sense for your specific use case (e.g., fiscal year starts, survey
periods, etc.).

### Roman Numerals

`datefixR` can handle Roman numerals being used to denote the month, a
format sometimes used by Oracle Database:

``` r
# Roman numeral months
roman_dates <- c(
  "15.VII.2023", # July 15, 2023
  "3.XII.1999", # December 3, 1999
  "1.I.2000" # January 1, 2000
)

fix_date_char(roman_dates, roman.numeral = TRUE)
#> [1] "2023-07-15" "1999-12-03" "2000-01-01"
```

Roman numeral support is experimental and has to be explicitly enabled
via `roman.numeral = TRUE`.

## Performance

This package has recently been optimized for speed using Rust and is now
over 300x faster than the largely pure R implementation used in previous
versions. Moreover, a fastpath approach has been implemented for common
date formats, further improving performance in most situations. Finally,
`fix_date_df()` now supports parallelism over columns via the `cores`
argument (or via the `'Ncpus'` global option). As such, speed is now
very unlikely to be an issue when using `datefixR` on large datasets.

## Limitations

Date and time data are often reported together in the same variable
(known as “datetime”). However datetime formats are not supported by
`datefixR`. The current rationale is this package is mostly used to
handle dates entered via free text web forms and it is much less common
for both date and time to be reported together in this input method.
However, if there is significant demand for support for datetime data in
the future this may added.

## Similar packages to datefixR

Please note this section is not regularly updated and may be out-of-date
by the the time you read it.

### `lubridate`

[`lubridate::guess_formats()`](https://lubridate.tidyverse.org/reference/guess_formats.html)
can be used to guess a date format and
[`lubridate::parse_date_time()`](https://lubridate.tidyverse.org/reference/parse_date_time.html)
calls this function when it attempts to parse a vector into a POSIXct
date-time object. However:

1.  When a date fails to parse in `{lubridate}` then the user is simply
    told how many dates failed to parse. In `datefixR` the user is told
    the ID (assumed to be the first column by default but can be
    user-specified) corresponding to the date which failed to parse and
    reports the considered date: making it much easier to figure out
    which dates supplied failed to parse and why.
2.  When imputing a missing day or month, there is no user-control over
    this behavior. For example, when imputing a missing month, the user
    may wish to impute July, the middle of the year, instead of January.
    However, January will always be imputed in `{lubridate}`. In
    `datefixR`, this behavior can be controlled by the `month.impute`
    argument.
3.  These functions require all possible date formats to be specified in
    the `orders` argument, which may result in a date format not being
    considered if the user forgets to list one of the possible formats.
    By contrast, `datefixR` only needs a format to be specified if
    month-first is to be preferred over day-first when guessing a date.

However, `{lubridate}` of course excels in general date manipulation and
is an excellent tool to use alongside `datefixR`.

### `anytime`

An alternative function is
[`anytime::anydate()`](https://dirk.eddelbuettel.com/code/anytime.html)
which also attempts to convert dates to a consistent format (R’s
`{Date}`). However, `anydate()` does not allow the user to control how
missing days or months are imputed (defaulting to the 1st day/month) and
assumes dates with missing components are in YMD format.

``` r
suppressWarnings(anytime::anydate("May 1994"))
#> [1] NA
anytime::anydate("1994 May")
#> [1] "1994-05-01"
```

Two digit years are also not supported.

``` r
suppressWarnings(anytime::anydate("01/05/94"))
#> [1] NA
anytime::anydate("01/05/1994")
#> [1] "1994-01-05"
```

### `parsedate`

`parsedate::parse_date()` also attempts to solve the problem of handling
arbitrary dates and parses dates into the `POSIXct` type. Unfortunately,
`parse_date()` cannot handle years before 1970 – instead imputing the
year as the current year without any warnings being raised.

``` r
parsedate::parse_date("april 15 1969")
#> [1] "2026-04-15 UTC"
```

Moreover, `parse_date()` assumes dates are in MDY format and does not
allow the user to specify otherwise. However, `{parsedate}` has
excellent support for handling dates in ISO 8601 formats.

### `stringi`, `readr`, and `clock`

These packages all use [ICU
library](https://unicode-org.github.io/icu/userguide/format_parse/datetime/)
when parsing dates (via `stringi::stri_datetime_parse()`,
`readr::parse_date()`, or `clock::date_parse()`) and therefore all
behave very similarly. Notably, all of these functions require the date
format to be specified including specifying a priori if a date is
missing. Ultimately, this makes these packages unsuitable when numerous
dates in different formats must be parsed.

``` r
readr::parse_date("02/2010", "%m/%Y")
#> [1] "2010-02-01"
```

However, these packages have support for weekdays and months in around
211 locales whereas `datefixR` supports much fewer languages due to
support for additional languages needing to be implemented individually
and by hand.

**Trade-offs to consider:**

  - **`datefixR`**: Better error reporting, flexible imputation, handles
    mixed formats automatically. Fast.
  - **`lubridate`**: Requires format specification, limited imputation
    control  
  - **`stringi`/`readr`/`clock`**: Require exact format specification,
    supports 211 locales
  - **`anytime`**: Variable performance, no imputation support, silent
    failures

For messy, mixed-format data where usability and error handling are
priorities, `datefixR` shines. Now that the core logic is handled in
Rust, performance has improved significantly making it suitable for very
large datasets.

## Contributing to datefixR

If you are interested in contributing to `datefixR`, please read our
[contributing
guide](https://github.com/ropensci/datefixR/blob/main/.github/CONTRIBUTING.md).

Please note that this package is released with a [Contributor Code of
Conduct](https://ropensci.org/code-of-conduct/). By contributing to this
project, you agree to abide by its terms.

## Citation

If you use this package in your research, please consider citing
`datefixR`\! An up-to-date citation can be obtained by running

``` r
citation("datefixR")
```

# Note: Any variables prefixed with `.` are used for text
# replacement in the Makevars.in and Makevars.win.in

# check the packages MSRV first
source("tools/msrv.R")

# check DEBUG and NOT_CRAN environment variables
env_debug <- Sys.getenv("DEBUG")
env_not_cran <- Sys.getenv("NOT_CRAN")

# check if the vendored zip file exists
vendor_exists <- file.exists("src/rust/vendor.tar.xz")

is_not_cran <- env_not_cran != ""
is_debug <- env_debug != ""

if (is_debug) {
  # if we have DEBUG then we set not cran to true
  # CRAN is always release build
  is_not_cran <- TRUE
  message("Creating DEBUG build.")
}

if (!is_not_cran) {
  message("Building for CRAN.")
}

# we set cran flags only if NOT_CRAN is empty and if
# the vendored crates are present.
.cran_flags <- ifelse(
  !is_not_cran && vendor_exists,
  "-j 2 --offline",
  ""
)

# when DEBUG env var is present we use `--debug` build
.profile <- ifelse(is_debug, "", "--release")
.clean_targets <- ifelse(is_debug, "", "$(TARGET_DIR)")

# We specify this target when building for webR
webr_target <- "wasm32-unknown-emscripten"

# here we check if the platform we are building for is webr
is_wasm <- identical(R.version$platform, webr_target)

# print to terminal to inform we are building for webr
if (is_wasm) {
  message("Building for WebR")
}

# we check if we are making a debug build or not
# if so, the LIBDIR environment variable becomes:
# LIBDIR = $(TARGET_DIR)/{wasm32-unknown-emscripten}/debug
# this will be used to fill out the LIBDIR env var for Makevars.in
target_libpath <- if (is_wasm) "wasm32-unknown-emscripten" else NULL
cfg <- if (is_debug) "debug" else "release"

# used to replace @LIBDIR@
.libdir <- paste(c(target_libpath, cfg), collapse = "/")

# use this to replace @TARGET@
# we specify the target _only_ on webR
# there may be use cases later where this can be adapted or expanded
.target <- ifelse(is_wasm, paste0("--target=", webr_target), "")

# add panic exports and symbol optimization flags only for WASM builds via RUSTFLAGS
.panic_exports <- ifelse(
  is_wasm,
  "RUSTFLAGS=\"$RUSTFLAGS -C panic=abort -C opt-level=s -C lto=fat -C codegen-units=1 -C strip=symbols\" ",
  ""
)

# read in the Makevars.in file checking
is_windows <- .Platform[["OS.type"]] == "windows"

# if windows we replace in the Makevars.win.in
mv_fp <- ifelse(
  is_windows,
  "src/Makevars.win.in",
  "src/Makevars.in"
)

# set the output file
mv_ofp <- ifelse(
  is_windows,
  "src/Makevars.win",
  "src/Makevars"
)

# delete the existing Makevars{.win/.wasm}
if (file.exists(mv_ofp)) {
  message("Cleaning previous `", mv_ofp, "`.")
  invisible(file.remove(mv_ofp))
}

# read as a single string
mv_txt <- readLines(mv_fp)

# replace placeholder values
new_txt <- gsub("@CRAN_FLAGS@", .cran_flags, mv_txt) |>
  gsub("@PROFILE@", .profile, x = _) |>
  gsub("@CLEAN_TARGET@", .clean_targets, x = _) |>
  gsub("@LIBDIR@", .libdir, x = _) |>
  gsub("@TARGET@", .target, x = _) |>
  gsub("@PANIC_EXPORTS@", .panic_exports, x = _)

message("Writing `", mv_ofp, "`.")
con <- file(mv_ofp, open = "wb")
writeLines(new_txt, con, sep = "\n")
close(con)

message("`tools/config.R` has finished.")

.onAttach <- function(libname, pkgname) {
  multi.byte <- l10n_info()$MBCS
  if (!multi.byte) {
    packageStartupMessage(
      "The current locale does not support multibyte characters. ",
      "You may run into difficulties if any months are given as ",
      "non-English language names. \n"
    )
  }
}

#' @title Shiny application standardizing date data in csv or excel files
#' @description A shiny application which allows users to standardize dates
#'   using a graphical user interface (GUI). Most features of \code{datefixR}
#'   are supported including imputing missing date data. Data can be provided as
#'   CSV (comma-separated value) or XLSX (Excel) files. Processed datasets can
#'   be downloaded as CSV files. Please note, the dependencies for this app
#'   (\code{DT}, \code{htmltools}, \code{readxl}, and \code{shiny}) are not
#'   installed alongside \code{datefixR}. This allows \code{datefixR} to be
#'   installed on secure systems where these packages may not be allowed. If one
#'   of these dependencies is not installed on the system when this function is
#'   called, then the user will be given the option of installing them.
#' @param theme Color theme for shiny app. Either \code{"datefixR"}
#'   (\code{datefixR} colors) or \code{"none"}(default shiny app styling).
#' @return A shiny app.
#' @seealso The \code{\link[shiny]{shiny}} package.
#' @examples
#' \dontrun{
#' fix_date_app()
#' }
#' @export
fix_date_app <- function(theme = "datefixR") {
  rlang::check_installed(
    c("DT", "shiny", "readxl", "htmltools"),
    reason = "to use `fix_date_app()`"
  )

  if (!(theme %in% c("datefixR", "none"))) {
    stop("theme should be 'datefixR' or 'none' \n")
  }

  ui <- shiny::fluidPage(
    .html_head(theme = theme),
    htmltools::div(
      class = "simpleDiv",
      shiny::titlePanel("datefixR", "datefixR")
    ),
    shiny::fluidRow(
      shiny::column(
        3,
        shiny::wellPanel(
          shiny::fileInput(
            "datafile",
            "File Upload",
            accept = c(".csv", ".xlsx"),
            placeholder = ".xlsx (Excel) or .csv file"
          ),
          shiny::uiOutput("columns"),
          shiny::selectInput(
            "date.input",
            "Day of month to impute",
            choices = c(seq(1, 28), NA)
          ),
          shiny::selectInput(
            "month.input",
            "Month to impute",
            choices = c(seq(1, 12), NA),
            selected = 7
          ),
          shiny::selectInput(
            "format",
            "Assumed format",
            choices = c("dmy", "mdy")
          ),
          shiny::actionButton("do", "Refresh"),
          shiny::downloadButton("downloadData", "Download")
        )
      ),
      shiny::column(
        9,
        shiny::tabsetPanel(
          shiny::tabPanel(
            "Results",
            DT::DTOutput("df")
          ),
          shiny::tabPanel(
            "About this app",
            htmltools::tags$h3("Introduction"),
            htmltools::tags$img(
              src = paste0(
                "https://raw.githubusercontent.com",
                "/nathansam/datefixR/main/man/",
                "figures/logo.png"
              ),
              align = "right",
              width = "170"
            ),
            htmltools::tags$p(
              "Dates can be some of the most infuriating data types to work
              with: especially if the date formats (e.g dd/mm/yyyy or
              yyyy-mm-dd) have not been standardized across observations or
              are missing data such as day of the month. If dates have been
              provided via a free text webfrom, then this is a very likely
              scenario."
            ),
            htmltools::HTML(
              "<p>
                This app uses the
                <a href = 'https://www.constantine-cooke.com/datefixR/'>
                  <code>datefixR</code>
                </a>
                R package to parse dates in many different formats from either
                an xlsx (Excel) or csv file. No knowledge of R is required to
                use this app
              </p>"
            ),
            htmltools::HTML(
              "<p>
                 To use this app, please select <em>Browse</em> in the navbar on
                 the left hand side of the page and select either an  xlsx or
                 CSV file you wish to process. Once the upload is complete,
                 click the <em>Refresh</em> button to show the uploaded data.
                 You can then select the columns you want to fix. Clicking on
                 refresh once more will parse the dates and present the updated
                 dataset. This updated dataset can be downloaded using the
                 download button.
               </p>"
            ),
            htmltools::tags$p(
              "Please note, if you are using this app via an online hosting
              platform, such as on shinyapps.io, then your file will temporarily
              be stored by the hosting platform. However, no data should be
              stored persistently. Nevertheless, user discretion is advised, and
              any sensitive data should be used with a version of this app which
              is running locally."
            ),
            htmltools::h3("License"),
            htmltools::HTML(
              "<p>
                This app, like the rest of <code>datefixR</code>, is licensed
                under
                <a href =
                'https://github.com/nathansam/datefixR/blob/main/LICENSE.md'>
                  GPL-3
                </a>
                .
               </p>"
            ),
            htmltools::h3("Bug Reports"),
            htmltools::HTML(
              "
              <p>
                If you encounter any bugs or have a feature request, please
                create a GitHub Issue
                <a href = 'https://www.github.com/ropensci/datefixR/issues'>
                  here.
                </a>
              </p>"
            ),
            htmltools::tags$hr(),
            "Developed and maintained by",
            htmltools::tags$a(
              href = "https://www.github.com/nathansam",
              "Nathan Constantine-Cooke"
            )
          )
        )
      )
    )
  )
  # Run the application
  return(shiny::shinyApp(ui = ui, server = .server))
}


.html_head <- function(theme) {
  site <- "https://docs.ropensci.org/datefixR/" # replace with ropensci docs

  if (theme == "datefixR") {
    htmltools::tags$head(
      htmltools::tags$style(htmltools::HTML(
        "
        @import url('https://fonts.googleapis.com/css2?family=Roboto:wght@300&family=Zen+Tokyo+Zoo&display=swap');

  body {
    font-family: 'Roboto', sans-serif;
    padding: 10px;
  }

  .nav-tabs {
    padding-top: 5px;
  }

   a {
    color: #2a9d8f;
   }

  img {
    padding-left: 20px;
  }

  .tab-content{
    padding-top: 8px;
  }

  .simpleDiv {
    background: #2a9d8f;
    color: #e7c36a;
    display: -webkit-box;
    border: 0px;
    border-radius: 20px;
    font-family: 'Zen Tokyo Zoo', cursive;
    padding: 5px;
    border-bottom-right-radius: 0px;
    border-bottom-left-radius: 0px;
    border-bottom-width: 5px;
    border-bottom-color: #1b333a;
    border-bottom-style: solid;
  }

  hr {
    border-top: 2px solid #1b333a;
  }

  .simpleDiv h2 {
      font-size: 40px !important;
      float: none !important;
      margin: 0 auto !important;
    }

  .shiny-options-group {
    border-radius: 4px;
    padding: 5px;
    padding-left: 10px;
    background-color: #264653 ;
  }


  .well {
    background-color: #2a9d8f !important;
    color: white !important;
    border: 0px;
    height: 90vh;
    border-radius: 0px;
    border-bottom-left-radius: 20px;
    border-right-width: 5px;
    border-right-color: #1b333a;
    border-right-style: solid;
  }

  .col-sm-8 {
    padding-top: 20px !important;
  }

  .container-fluid {
    padding-right: 0px !important;
    padding-left: 0px !important;
  }

  .btn-default {
    background-color: #264653 !important;
    color: white !important;
  }"
      )),
      htmltools::tags$link(
        rel = "icon",
        type = "image/png",
        sizes = "16x16",
        href = paste0(site, "favicon-16x16.png")
      ),
      htmltools::tags$link(
        rel = "icon",
        type = "image/png",
        sizes = "32x32",
        href = paste0(site, "favicon-32x32.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "180x180",
        href = paste0(site, "apple-touch-icon.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "120x120",
        href = paste0(site, "apple-touch-icon-120x120.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "76x76",
        href = paste0(site, "apple-touch-icon-76x76.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "60x60",
        href = paste0(site, "apple-touch-icon-60x60.png")
      ),
      htmltools::tags$link(
        rel = "stylesheet",
        type = "text/css",
        href = "style.css"
      )
    )
  } else {
    htmltools::tags$head(
      htmltools::tags$link(
        rel = "icon",
        type = "image/png",
        sizes = "16x16",
        href = paste0(site, "favicon-16x16.png")
      ),
      htmltools::tags$link(
        rel = "icon",
        type = "image/png",
        sizes = "32x32",
        href = paste0(site, "favicon-32x32.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "180x180",
        href = paste0(site, "apple-touch-icon.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "120x120",
        href = paste0(site, "apple-touch-icon-120x120.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "76x76",
        href = paste0(site, "apple-touch-icon-76x76.png")
      ),
      htmltools::tags$link(
        rel = "apple-touch-icon",
        type = "image/png",
        sizes = "60x60",
        href = paste0(site, "apple-touch-icon-60x60.png")
      ),
      htmltools::tags$link(
        rel = "stylesheet",
        type = "text/css",
        href = "style.css"
      )
    )
  }
}

.server <- function(input, output) {
  output.data <- NULL
  shiny::observeEvent(input$do, {
    input.data <- .read_data(input$datafile)
    if (!any(is.null(input$selected.columns))) {
      output.data <- fix_date_df(
        input.data,
        col.names = input$selected.columns,
        day.impute = as.numeric(input$date.input),
        month.impute = as.numeric(input$month.input)
      )
    }
    if (is.null(output.data)) {
      output.data <- input.data
    }
    output$df <- DT::renderDT({
      output.data
    })
    output$columns <- shiny::renderUI({
      shiny::checkboxGroupInput(
        "selected.columns",
        "",
        colnames(input.data)
      )
    })
    output$downloadData <- shiny::downloadHandler(
      filename = function() {
        "fixed.csv"
      },
      content = function(file) {
        input.data <- .read_data(input$datafile)
        if (!any(is.null(input$selected.columns))) {
          output.data <- fix_date_df(
            input.data,
            col.names = input$selected.columns
          )
        }
        utils::write.csv(output.data, file)
      }
    )
  })
}

.read_data <- function(upload) {
  if (!is.null(upload)) {
    ext <- tools::file_ext(upload$datapath)
    if (ext == "csv") {
      return((utils::read.csv(upload$datapath)))
    } else {
      return(as.data.frame(readxl::read_xlsx(upload$datapath)))
    }
  }
}

# This file is for testing the applications in the apps/ directory.

test_that("fix_date_app() works with datefixR theme", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()
  skip_if_not(
    Sys.info()["sysname"] == "Darwin",
    message = "These tests only run on macOS"
  )
  withr::local_package("shinytest2")
  shiny_app <- fix_date_app()
  app <- AppDriver$new(shiny_app, variant = platform_variant())
  app$set_window_size(width = 779, height = 853)
  # Uploaded file outside of: ./tests/testthat
  app$upload_file(datafile = system.file("example.xlsx", package = "datefixR"))
  app$click("do")
  # Update output value
  app$set_inputs(
    df_rows_current = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app$set_inputs(
    df_rows_all = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app$set_inputs(
    df_state = c(
      1673953581559,
      0,
      10,
      "",
      TRUE,
      FALSE,
      TRUE,
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE)
    ),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app$set_inputs(`selected.columns` = "Apple")
  app$set_inputs(`selected.columns` = c("Apple", "Pear"))
  app$click("do")
  app$expect_values(output = TRUE)
  # Update output value
  app$set_inputs(
    df_rows_current = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app$set_inputs(
    df_rows_all = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app$set_inputs(
    df_state = c(
      1673953593973,
      0,
      10,
      "",
      TRUE,
      FALSE,
      TRUE,
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE)
    ),
    allow_no_input_binding_ = TRUE
  )
  app$expect_download("downloadData")
  app$set_window_size(width = 779, height = 796)
  app$expect_values(output = TRUE)
})

test_that("fix_date_app() works with shiny theme", {
  # Don't run these tests on the CRAN build servers
  skip_on_cran()
  skip_if_not(
    Sys.info()["sysname"] == "Darwin",
    message = "These tests only run on macOS"
  )
  withr::local_package("shinytest2")
  shiny_app <- fix_date_app(theme = "none")
  app2 <- AppDriver$new(shiny_app, variant = "shiny")
  app2$set_window_size(width = 779, height = 853)
  # Uploaded file outside of: ./tests/testthat
  app2$upload_file(datafile = system.file("example.csv", package = "datefixR"))
  app2$click("do")
  # Update output value
  app2$set_inputs(
    df_rows_current = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app2$set_inputs(
    df_rows_all = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app2$set_inputs(
    df_state = c(
      1673953581559,
      0,
      10,
      "",
      TRUE,
      FALSE,
      TRUE,
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE)
    ),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app2$set_inputs(`selected.columns` = "Apple")
  app2$set_inputs(`selected.columns` = c("Apple", "Pear"))
  app2$click("do")
  app2$expect_values(output = TRUE)
  # Update output value
  app2$set_inputs(
    df_rows_current = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app2$set_inputs(
    df_rows_all = c(1, 2, 3, 4),
    allow_no_input_binding_ = TRUE,
    wait_ = FALSE
  )
  app2$set_inputs(
    df_state = c(
      1673953593973,
      0,
      10,
      "",
      TRUE,
      FALSE,
      TRUE,
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE),
      c(TRUE, "", TRUE, FALSE, TRUE)
    ),
    allow_no_input_binding_ = TRUE
  )
  app2$expect_download("downloadData")
  app2$set_window_size(width = 779, height = 796)
  app2$expect_values(output = TRUE)
})

test_that("unexpected theme raises error raises error", {
  expect_error(
    fix_date_app(theme = "test"),
    "theme should be 'datefixR' or 'none' \n"
  )
})

test_that(".read_data works for xlsx files", {
  upload <- list()
  upload$datapath <- system.file("example.xlsx", package = "datefixR")

  exp.data <- data.frame(
    Apple = c(
      "02-03-21",
      "05 02 19",
      "April 2019",
      "12 1993"
    ),
    Pear = c(
      "Jan 2000",
      "1990",
      "sept 1984",
      "1990-05-01"
    ),
    Lemon = c(
      "1 1 2010",
      "2 Jan 1990",
      "25 Mar 1975",
      "December 2020"
    )
  )
  expect_equal(datefixR:::.read_data(upload), exp.data)
})


test_that(".read_data works for csv files", {
  upload <- list()
  upload$datapath <- system.file("example.xlsx", package = "datefixR")

  exp.data <- data.frame(
    Apple = c(
      "02-03-21",
      "05 02 19",
      "April 2019",
      "12 1993"
    ),
    Pear = c(
      "Jan 2000",
      "1990",
      "sept 1984",
      "1990-05-01"
    ),
    Lemon = c(
      "1 1 2010",
      "2 Jan 1990",
      "25 Mar 1975",
      "December 2020"
    )
  )
  expect_equal(datefixR:::.read_data(upload), exp.data)
})

#' datefixR: Standardize Dates in Different Formats or with Missing Data
#'
#' @description There are many different formats dates are commonly represented
#'   with: the order of day, month, or year can differ, different separators
#'   ("-", "/", or whitespace) can be used, months can be numerical, names, or
#'   abbreviations and year given as two digits or four. `datefixR` takes dates
#'   in all these different formats and converts them to \R{}'s built-in date
#'   class. If `datefixR` cannot standardize a date, such as because it is too
#'   malformed, then the user is told which date cannot be standardized and the
#'   corresponding ID for the row. `datefixR` also allows the imputation of
#'   missing days and months with user-controlled behavior.
#'
#'   Get started by reading \code{vignette("datefixR")}
#'
#' @seealso
#' Useful links:
#' * \url{https://docs.ropensci.org/datefixR/}
#' * \url{https://github.com/ropensci/datefixR/}
#' * Report bugs at \url{https://github.com/ropensci/datefixR/issues}
## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @useDynLib datefixR, .registration = TRUE
## usethis namespace: end
#' @docType package
#' @name datefixR
#' @keywords internal
"_PACKAGE"

#' @title Clean up messy date columns
#' @description Tidies a \code{dataframe} or \code{tibble} object with date
#' columns entered via a free-text interface, addressing non-standardized
#' formats. Supports diverse separators including /, -, ., and spaces. Handles
#' all-numeric, abbreviated, or full-length month names in languages such as
#' English, French, German, Spanish,  Portuguese, Russian, Czech, Slovak, and
#' Indonesian. Imputes missing day data by  default, with flexibility for custom
#' imputation strategies.
#'
#' @details
#' This function processes messy date data by:
#' \itemize{
#'   \item{Supporting mixed format data entries}
#'   \item{Recognizing multilingual month names and Roman numeral inputs}
#'   \item{Interpreting Excel-style serial date numbers if specified}
#'   \item{Providing warnings and controls for missing day/month imputation}
#' }
#' For further details and advanced usage, refer to the vignette via
#' \code{browseVignettes("datefixR")} or visit the online documentation at
#' \url{https://docs.ropensci.org/datefixR/}.
#'
#' @param df A \code{dataframe} or \code{tibble} object containing messy date
#'   column(s).
#' @param col.names Character vector specifying column names of date data to be
#'   cleaned.
#' @param id Optional parameter specifying the name of the column containing
#'   row IDs. Defaults to using the first column for IDs.
#' @param day.impute Integer between 1 and 31, or NA, or NULL. Day of the month
#'   to be imputed when missing. Defaults to 1. If \code{day.impute} is greater
#'   than the number of days in a given month, the last day of that month will
#'   be imputed (accounting for leap years). If \code{day.impute = NA}, then
#'   \code{NA} will be imputed for the entire date and a warning will be raised.
#'   If \code{day.impute = NULL}, the function will fail with an error when day
#'   is missing.
#' @param month.impute Integer between 1 and 12, or NA, or NULL. Month to be
#'   imputed when missing. Defaults to 7 (July). If \code{month.impute = NA},
#'   then \code{NA} will be imputed for the entire date and a warning will be
#'   raised.
#'   If \code{month.impute = NULL}, the function will fail with an error when
#'   month is missing.
#' @param format Character string specifying date interpretation preference.
#'   Either \code{"dmy"} (day-month-year, default) or \code{"mdy"}
#'   (month-day-year, US format). This setting only affects ambiguous numeric
#'   dates like "01/02/2023". When month names are present or year appears
#'   first, the format is auto-detected regardless of this parameter. Note that
#'   unambiguous dates (e.g., "25/12/2023") are parsed correctly regardless of
#'   the format setting.
#' @param excel Logical: Assumes \code{FALSE} by default. If \code{TRUE}, treats
#'   numeric-only dates with more than four digits as Excel serial dates with
#'   1900-01-01 origin, correcting for known Excel date discrepancies.
#' @param roman.numeral `r lifecycle::badge("experimental")` Logical: Defaults
#'   to \code{FALSE}. When \code{TRUE}, attempts to interpret Roman numeral
#'   month indications within datasets. This feature may not handle all cases
#'   correctly.
#' @param cores Integer: Number of CPU cores to use for parallel processing.
#'   Defaults to \code{getOption("Ncpus", 1)}. When \code{cores > 1}, processes
#'   multiple date columns in parallel using the \code{future} framework.
#'   Requires the \code{future} and \code{future.apply} packages to be installed.
#'   The actual number of workers used will be the minimum of \code{cores} and
#'   the number of columns to process.
#' @return A revised \code{dataframe} or \code{tibble} structure, maintaining
#'   input type. Date columns will be formatted with \code{Date} class and
#'   display as \code{yyyy-mm-dd}.
#' @seealso
#' \code{\link{fix_date_char}} for similar functionality on character vectors.
#'
#' For comprehensive examples and usage practices, consult:
#' \itemize{
#'   \item{Vignette: \code{browseVignettes("datefixR")}}
#'   \item{Documentation: \url{https://docs.ropensci.org/datefixR/articles/datefixR.html}}
#'   \item{README Overview: \url{https://docs.ropensci.org/datefixR/}}
#' }
#' @examples
#' # Basic cleanup
#' data(exampledates)
#' fix_date_df(exampledates, c("some.dates", "some.more.dates"))
#'
#' # Usage with metadata
#' messy_dates_df <- data.frame(
#'   id = seq(1, 3),
#'   dates = c("1992", "April 1990", "Mar 19")
#' )
#' fix_date_df(messy_dates_df, "dates", day.impute = 15, month.impute = 12)
#'
#' # Diverse format normalization
#' df_formats <- data.frame(
#'   mixed.dates = c("02/05/92", "2020-may-01", "1996.05.01", "October 2022"),
#'   european.dates = c("22.07.1977", "05.06.2023")
#' )
#' fix_date_df(df_formats, c("mixed.dates", "european.dates"))
#'
#' # Excel serial examples
#' serial_df <- data.frame(serial.dates = c("44197", "44927"))
#' fix_date_df(serial_df, "serial.dates", excel = TRUE)
#'
#' # Handling Roman numerals
#' roman_df <- data.frame(roman.dates = c("15.I.2023", "03.XII.2019"))
#' fix_date_df(roman_df, "roman.dates", roman.numeral = TRUE)
#'
#' # Parallel processing (requires 'future' and 'future.apply' packages)
#' \dontrun{
#' large_df <- data.frame(
#'   dates1 = c("01/02/2020", "15/03/2021", "22/12/2019"),
#'   dates2 = c("2020-01-01", "March 2021", "Dec 2019"),
#'   dates3 = c("01.01.20", "15.03.21", "22.12.19")
#' )
#' # Use 4 cores for parallel processing
#' fix_date_df(large_df, c("dates1", "dates2", "dates3"), cores = 4)
#'
#' # Use all available cores (respects getOption("Ncpus"))
#' options(Ncpus = parallel::detectCores())
#' fix_date_df(large_df, c("dates1", "dates2", "dates3"))
#' }
#' @export
fix_date_df <- function(
  df,
  col.names,
  day.impute = 1,
  month.impute = 7,
  id = NULL,
  format = "dmy",
  excel = FALSE,
  roman.numeral = FALSE,
  cores = getOption("Ncpus", 1)
) {
  if (!is.data.frame(df)) {
    stop("df should be a dataframe object!")
  }
  if (any(!is.character(col.names))) {
    stop("col.names should be a character vector!")
  }
  .checkformat(format)

  if (is.null(id)) {
    id <- 1
  } # Use first column as id if not explicitly given

  # Note: NA values in input data are handled by the Rust backend and converted to NA dates

  # Handle NULL values first - these should throw errors immediately
  if (is.null(day.impute)) {
    stop("Missing day with no imputation value given \n", call. = FALSE)
  }
  if (is.null(month.impute)) {
    stop("Missing month with no imputation value given \n", call. = FALSE)
  }

  # Check day.impute for valid values (but only if not NA)
  if (!is.na(day.impute)) {
    checkday_result <- checkday(day.impute)
    if (inherits(checkday_result, "extendr_error")) {
      error_msg <- if ("value" %in% names(checkday_result)) {
        checkday_result$value
      } else {
        as.character(checkday_result)
      }
      stop(error_msg, call. = FALSE)
    }
  }

  .checkmonth(month.impute)

  # Convert imputation values to integers for Rust
  # Pass -1 as a sentinel value for NA, which Rust will interpret as a special case
  day_impute_int <- if (is.na(day.impute)) -1L else as.integer(day.impute)
  month_impute_int <- if (is.na(month.impute)) -1L else as.integer(month.impute)

  # Function to process a single column
  process_column <- function(col.name) {
    # Pre-process data to handle NA and empty values
    date_data <- as.character(df[[col.name]])
    na_indices <- is.na(date_data) | date_data == "" | date_data == "NA"

    # Only process non-NA values through Rust
    if (all(na_indices)) {
      # All values are NA, return all NAs
      fixed.dates <- rep(list(NULL), length(date_data))
    } else {
      # Replace NA values with placeholder for processing
      processed_data <- date_data
      processed_data[na_indices] <- "1999-01-01" # Temporary placeholder

      subjects <- if (is.numeric(id) && id <= ncol(df)) as.character(df[[id]]) else NULL

      # Call Rust backend
      fixed.dates <- .Call(
        "wrap__fix_date_column",
        processed_data,
        day_impute_int,
        month_impute_int,
        subjects,
        format,
        excel,
        roman.numeral
      )

      # Generate warnings for NA imputation when day.impute or month.impute is NA
      day_is_na <- !is.null(day.impute) && is.na(day.impute)
      month_is_na <- !is.null(month.impute) && is.na(month.impute)
      if (day_is_na || month_is_na) {
        for (i in seq_along(fixed.dates)) {
          if (is.null(fixed.dates[[i]]) || is.na(fixed.dates[[i]])) {
            # This date resulted in NA, generate appropriate warning
            original_date <- date_data[i] # Use original date data, not processed
            subject_id <- if (!is.null(subjects)) subjects[i] else i

            if (day_is_na) {
              warning(sprintf("NA imputed for subject %s (date: %s)", subject_id, original_date), call. = FALSE)
            } else if (month_is_na) {
              warning(sprintf("NA imputed for subject %s (date: %s)", subject_id, original_date), call. = FALSE)
            }
          }
        }
      }

      # Restore NA values in the result
      if (is.list(fixed.dates)) {
        fixed.dates[na_indices] <- list(NULL)
      } else if (is.vector(fixed.dates)) {
        # Handle case where fixed.dates is a vector
        fixed.dates[na_indices] <- NA
      }
    }

    # Check if the result is an error condition from extendr
    if (inherits(fixed.dates, "extendr_error")) {
      # Extract the error message and throw it as a proper R error
      error_msg <- if ("value" %in% names(fixed.dates)) {
        fixed.dates$value
      } else {
        as.character(fixed.dates)
      }
      stop(error_msg, call. = FALSE)
    }

    # Convert to Date class, handling NULL values from Rust
    return(as.Date(sapply(fixed.dates, function(x) if (is.null(x)) NA_character_ else x)))
  }

  # Process columns either in parallel or sequentially
  if (cores > 1 && length(col.names) > 1) {
    # Check if required packages are available
    if (!requireNamespace("future", quietly = TRUE) || !requireNamespace("future.apply", quietly = TRUE)) {
      stop("Parallel processing requires 'future' and 'future.apply' packages. Install with: install.packages(c('future', 'future.apply'))")
    }

    # Determine optimal number of workers (minimum of cores and columns)
    n_workers <- min(cores, length(col.names))

    # Set up parallel plan
    future::plan(future::multisession, workers = n_workers)

    # Process columns in parallel
    result_columns <- future.apply::future_lapply(col.names, process_column)
    names(result_columns) <- col.names

    # Assign results back to dataframe
    for (i in seq_along(col.names)) {
      df[, col.names[i]] <- result_columns[[i]]
    }
  } else {
    # Process columns sequentially
    for (col.name in col.names) {
      df[, col.name] <- process_column(col.name)
    }
  }
  # Reset future plan to sequential after processing (closes workers)
  future::plan(future::sequential)

  df
}

test_that("fix_date_df works for a series of malformed dates", {
  bad.dates <- data.frame(
    id = seq(7),
    some.dates = c(
      "02/05/92",
      "01-04-2020",
      "1996/05/01",
      "2020-05-01",
      "02-04-96",
      "2015",
      "Sept 2009"
    ),
    some.more.dates = c(
      "02/05/00",
      "05/1990",
      "2012-08",
      "apr-21",
      "mar-65",
      "feb 84",
      "5 sept 2021"
    )
  )

  fixed.df <- fix_date_df(bad.dates, c("some.dates", "some.more.dates"))

  expected.df <- data.frame(
    id = seq(7),
    some.dates = c(
      "1992-05-02",
      "2020-04-01",
      "1996-05-01",
      "2020-05-01",
      "1996-04-02",
      "2015-07-01",
      "2009-09-01"
    ),
    some.more.dates = c(
      "2000-05-02",
      "1990-05-01",
      "2012-08-01",
      "2021-04-01",
      "1965-03-01",
      "1984-02-01",
      "2021-09-05"
    )
  )
  expected.df$some.dates <- as.Date(expected.df$some.dates)
  expected.df$some.more.dates <- as.Date(expected.df$some.more.dates)
  expect_equal(fixed.df, expected.df)
})

test_that("returns error for dates that are malformed beyond auto fix", {
  bad.dates <- data.frame(
    id = seq(9),
    some.dates = c(
      "02/05/92",
      "01-04-2020",
      "1996/05/01",
      "2020-05-01",
      "02-04-96",
      "2015",
      "02/05/00",
      "05/1990",
      "20125/02"
    )
  )
  expect_error(fix_date_df(bad.dates, "some.dates"), "unable to tidy a date")
})

test_that("Handle empty dates correctly", {
  really.bad.dataframe <- data.frame(
    id = seq(2),
    some.dates = c(NA, "")
  )
  fixed.df <- fix_date_df(really.bad.dataframe, "some.dates")

  expected.df <- data.frame(
    id = seq(2),
    some.dates = c(NA, NA)
  )
  expected.df$some.dates <- as.Date(expected.df$some.dates)

  expect_equal(fixed.df, expected.df)
})

test_that("error if unexpected data type", {
  exvector <- c(1, 2, 3)
  expect_error(
    fix_date_df(exvector, "some.dates"),
    "df should be a dataframe object!"
  )

  expect_error(
    fix_date_df(data.frame(), 3),
    "col.names should be a character vector!"
  )
})

test_that("error if day.impute or month.impute are wrong format", {
  exvector <- c(1, 2, 3)
  expect_error(
    fix_date_df(exvector, "some.dates"),
    "df should be a dataframe object!"
  )

  expect_error(
    fix_date_df(data.frame(), 3),
    "col.names should be a character vector!"
  )
  bad.dates <- data.frame(
    id = seq(6),
    some.dates = c(
      "02/05/92",
      "01-04-2020",
      "1996/05/01",
      "2020-05-01",
      "02-04-96",
      "2015"
    ),
    some.more.dates = c(
      "02/05/00",
      "05/1990",
      "2012-08",
      "apr-21",
      "mar-65",
      "feb 84"
    )
  )

  expect_error(
    fix_date_df(bad.dates, c("some.dates", "some.more.dates"), day.impute = 35),
    "day.impute should be an integer between 1 and 31\n"
  )
  expect_error(
    fix_date_df(
      bad.dates,
      c("some.dates", "some.more.dates"),
      day.impute = 2.2
    ),
    "day.impute should be an integer\n"
  )
  expect_error(
    fix_date_df(
      bad.dates,
      c("some.dates", "some.more.dates"),
      month.impute = 13
    ),
    "month.impute should be an integer between 1 and 12\n"
  )
  expect_error(
    fix_date_df(
      bad.dates,
      c("some.dates", "some.more.dates"),
      month.impute = 2.2
    ),
    "month.impute should be an integer\n"
  )
  expect_error(
    fix_date_df(
      bad.dates,
      c("some.dates", "some.more.dates"),
      month.impute = "apr"
    ),
    "month.impute should be an integer between 1 and 12\n"
  )
})

test_that("error if day.impute or month.impute are in wrong format", {
  temp <- data.frame(example = "1994")

  expect_equal(
    fix_date_df(temp, "example", month.impute = 11),
    data.frame(example = as.Date("1994-11-01"))
  )
})

test_that("Error if month out of bounds", {
  temp <- data.frame(id = 1, date = "13-1994")

  expect_error(
    fix_date_df(temp, "date"),
    "Month not in expected range"
  )
})


test_that("Imputing day with NA results in NA", {
  temp <- data.frame(id = 1, date = "04-1994")
  expect_warning(
    date.guess <- fix_date_df(temp, "date", day.impute = NA),
    "NA imputed for subject 1 \\(date: 04\\-1994\\)"
  )
  expect_equal(date.guess, data.frame(id = 1, date = as.Date(NA)))
})


test_that("Imputing month with NA results in NA", {
  temp <- data.frame(id = 1, date = "1994")
  expect_warning(
    date.guess <- fix_date_df(temp, "date", month.impute = NA),
    "NA imputed for subject 1 \\(date: 1994\\)"
  )
  expect_equal(
    date.guess,
    data.frame(id = 1, date = as.Date(NA))
  )
})


test_that("Error if imputing month with NULL", {
  temp <- data.frame(id = 1, date = "1994")
  expect_error(
    fix_date_df(temp, "date", month.impute = NULL),
    "Missing month with no imputation value given \n"
  )
})

test_that("Error if imputing day with NULL", {
  temp <- data.frame(id = 1, date = "07-1994")
  expect_error(
    fix_date_df(temp, "date", day.impute = NULL),
    "Missing day with no imputation value given \n"
  )
})


test_that("fix_date_df works for a mdy format", {
  bad.dates <- data.frame(
    id = seq(6),
    some.dates = c(
      "05/02/92",
      "04-01-2020",
      "1996/05/01",
      "2020-01-05",
      "04-02-96",
      "2015"
    ),
    some.more.dates = c(
      "05/02/00",
      "05/1990",
      "2012-08",
      "apr-21",
      "mar-65",
      "feb 84"
    )
  )

  fixed.df <- fix_date_df(
    bad.dates,
    c("some.dates", "some.more.dates"),
    format = "mdy"
  )

  expected.df <- data.frame(
    id = seq(6),
    some.dates = c(
      "1992-05-02",
      "2020-04-01",
      "1996-05-01",
      "2020-01-05",
      "1996-04-02",
      "2015-07-01"
    ),
    some.more.dates = c(
      "2000-05-02",
      "1990-05-01",
      "2012-08-01",
      "2021-04-01",
      "1965-03-01",
      "1984-02-01"
    )
  )
  expected.df$some.dates <- as.Date(expected.df$some.dates)
  expected.df$some.more.dates <- as.Date(expected.df$some.more.dates)
  expect_equal(fixed.df, expected.df)
})


test_that("Non-excel nuneric date is parsed correctly", {
  bad.date <- data.frame(id = 1, some.date = "19374")
  fixed.df <- fix_date_df(bad.date, "some.date")
  expected.df <- data.frame(id = 1, some.date = "2023-01-17")
  expected.df$some.date <- as.Date(expected.df$some.date)
  expect_equal(fixed.df, expected.df)
})

test_that("Excel numeric date is parsed correctly", {
  bad.date <- data.frame(id = 1, some.date = "41035")
  fixed.df <- fix_date_df(bad.date, "some.date", excel = TRUE)
  expected.df <- data.frame(id = 1, some.date = "2012-05-06")
  expected.df$some.date <- as.Date(expected.df$some.date)
  expect_equal(fixed.df, expected.df)
})

test_that("checkday errors when input is out of range", {
  result <- try(datefixR:::checkday(45), silent = TRUE)
  expect_s3_class(result, "extendr_error")
  # Note: The actual error message is wrapped in try-error and not directly accessible

  # Test that valid values work
  expect_no_condition(datefixR:::checkday(15)) # Should return without error for valid day
  expect_no_condition(datefixR:::checkday(1)) # Should return without error for valid day
  expect_no_condition(datefixR:::checkday(31)) # Should return without error for valid day

  # Test other invalid values also throw errors
  expect_s3_class(try(datefixR:::checkday(0), silent = TRUE), "extendr_error")
  expect_s3_class(try(datefixR:::checkday(32), silent = TRUE), "extendr_error")
})


test_that("Roman numerical dates handled correctly", {
  bad.dataframe <- data.frame(
    id = seq(2),
    some.dates = c("20-iv-2023", "2010/v/19")
  )
  fixed.df <- fix_date_df(bad.dataframe, "some.dates", roman.numeral = TRUE)

  expected.df <- data.frame(
    id = seq(2),
    some.dates = c("2023-04-20", "2010-05-19")
  )
  expected.df$some.dates <- as.Date(expected.df$some.dates)

  expect_equal(fixed.df, expected.df)
})

test_that("parallel processing with multiple date columns works correctly", {
  # Skip if future packages are not available
  skip_if_not_installed("future")
  skip_if_not_installed("future.apply")

  # Create test dataframe with two date columns
  test_df <- data.frame(
    id = 1:4,
    dates_col1 = c("01/01/2020", "02/02/2021", "03/03/2022", "15-12-2019"),
    dates_col2 = c("04/04/2020", "05/05/2021", "06/06/2022", "dec 2018")
  )

  # Process with parallel processing (2 cores)
  result_parallel <- fix_date_df(test_df, c("dates_col1", "dates_col2"), cores = 2)

  # Process sequentially for comparison
  result_sequential <- fix_date_df(test_df, c("dates_col1", "dates_col2"), cores = 1)

  # Results should be identical regardless of processing method
  expect_equal(result_parallel, result_sequential)

  # Verify the actual parsed dates are correct
  expected_df <- data.frame(
    id = 1:4,
    dates_col1 = as.Date(c("2020-01-01", "2021-02-02", "2022-03-03", "2019-12-15")),
    dates_col2 = as.Date(c("2020-04-04", "2021-05-05", "2022-06-06", "2018-12-01"))
  )

  expect_equal(result_parallel, expected_df)
})

test_that("parallel processing falls back to sequential for single column", {
  # Create test dataframe with one date column
  test_df <- data.frame(
    id = 1:3,
    single_date_col = c("01/01/2020", "02/02/2021", "03/03/2022")
  )

  # Process with cores = 4 (should fall back to sequential since only 1 column)
  result <- fix_date_df(test_df, "single_date_col", cores = 4)

  # Verify the result is correct
  expected_df <- data.frame(
    id = 1:3,
    single_date_col = as.Date(c("2020-01-01", "2021-02-02", "2022-03-03"))
  )

  expect_equal(result, expected_df)
})

test_that("NA values in dataframe columns are preserved correctly", {
  # Test dataframe with mixed NA, empty string, and "NA" string
  test_df <- data.frame(
    id = 1:5,
    dates = c("2023-01-01", NA, "2024-01-01", "", "NA")
  )

  result_df <- fix_date_df(test_df, "dates")

  # Check that valid dates are parsed correctly
  expect_equal(result_df$dates[1], as.Date("2023-01-01"))
  expect_equal(result_df$dates[3], as.Date("2024-01-01"))

  # Check that all types of NA are preserved as NA
  expect_true(is.na(result_df$dates[2])) # NA
  expect_true(is.na(result_df$dates[4])) # empty string
  expect_true(is.na(result_df$dates[5])) # "NA" string

  # Ensure we didn't get the placeholder date (1999-01-01)
  expect_false(any(result_df$dates == as.Date("1999-01-01"), na.rm = TRUE))
})

test_that("parse error includes subject ID and date", {
  temp <- data.frame(id = 5, date = "20125/02")
  expect_error(
    fix_date_df(temp, "date"),
    "for subject 5 \\(date: 20125/02\\)"
  )
})

test_that("parse error reports correct subject when error is not in first row", {
  temp <- data.frame(
    id = c(1, 2, 3),
    date = c("01/01/2020", "02/02/2021", "20125/02")
  )
  expect_error(
    fix_date_df(temp, "date"),
    "for subject 3 \\(date: 20125/02\\)"
  )
})

test_that("Dataframe with all NA values returns all NA", {
  test_df <- data.frame(
    id = 1:3,
    dates = c(NA, "", "NA")
  )

  result_df <- fix_date_df(test_df, "dates")

  # All should be NA
  expect_true(all(is.na(result_df$dates)))
  expect_equal(length(result_df$dates), 3)

  # Ensure we didn't get the placeholder date
  expect_false(any(result_df$dates == as.Date("1999-01-01"), na.rm = TRUE))
})

#' @title Convert non-standardized dates to \R{}'s \code{Date} class
#' @description
#' Converts a character vector (or single character object) from inconsistently
#' formatted dates to \R{}'s \code{Date} class. Supports numerous separators
#' including /, -, ., or space. Supports numeric, abbreviation, or long-hand
#' month notation in multiple languages (English, French, German, Spanish,
#' Portuguese, Russian, Czech, Slovak, Indonesian). Where day of the month has
#' not been supplied, the first day of the month is imputed by default. Either
#' DMY or YMD is assumed by default. However, the US system of MDY is supported
#' via the \code{format} argument.
#'
#' @details
#' This function intelligently parses dates by:
#' \itemize{
#'   \item{Handling mixed separators within the same dataset}
#'   \item{Recognizing month names in multiple languages}
#'   \item{Converting Roman numeral months (experimental)}
#'   \item{Processing Excel serial date numbers}
#'   \item{Automatically detecting YMD format when year appears first}
#'   \item{Smart imputation of missing date components with user control}
#' }
#'
#' For comprehensive examples and advanced usage, see \code{browseVignettes("datefixR")}
#' or the package README at \url{https://docs.ropensci.org/datefixR/}.
#'
#' @param dates Character vector to be converted to \R{}'s date class.
#' @param day.impute Integer between 1 and 31, or NA, or NULL. Day of the month
#'   to be imputed when missing. Defaults to 1. If \code{day.impute = NA}, then
#'   \code{NA} will be imputed for the date and a warning will be raised. If
#'   \code{day.impute = NULL}, the function will fail with an error when day is
#'   missing.
#' @param month.impute Integer between 1 and 12, or NA, or NULL. Month to be
#'   imputed when missing. Defaults to 7 (July). If \code{month.impute = NA},
#'   then \code{NA} will be imputed for the entire date and a warning will be
#'   raised.
#'   If \code{month.impute = NULL}, the function will fail with an error when
#'   month is missing.
#' @param format Character string specifying date interpretation preference.
#'   Either \code{"dmy"} (day-month-year, default) or \code{"mdy"}
#'   (month-day-year, US format). This setting only affects ambiguous numeric
#'   dates like "01/02/2023". When month names are present or year appears
#'   first, the format is auto-detected regardless of this parameter. Note that
#'   unambiguous dates (e.g., "25/12/2023") are parsed correctly regardless of
#'   the format setting.
#' @inheritParams fix_date_df
#' @return A vector of elements belonging to \R{}'s built in \code{Date} class
#'   with the following format \code{yyyy-mm-dd}.
#' @seealso
#' \code{\link{fix_date_df}} for data frame columns with date data.
#'
#' For detailed examples and usage patterns, see:
#' \itemize{
#'   \item{Package vignette: \code{browseVignettes("datefixR")}}
#'   \item{Online documentation: \url{https://docs.ropensci.org/datefixR/articles/datefixR.html}}
#'   \item{Package README: \url{https://docs.ropensci.org/datefixR/}}
#' }
#' @examples
#' # Basic usage
#' bad.date <- "02 03 2021"
#' fix_date_char(bad.date)
#'
#' # Multiple formats with different separators
#' mixed_dates <- c(
#'   "02/05/92", # slash separator, 2-digit year
#'   "2020-may-01", # hyphen separator, text month
#'   "1996.05.01", # dot separator
#'   "02 04 96", # space separator
#'   "le 3 mars 2013" # French format
#' )
#' fix_date_char(mixed_dates)
#'
#' # Text months in different languages
#' text_months <- c(
#'   "15 January 2020", # English
#'   "15 janvier 2020", # French
#'   "15 Januar 2020", # German
#'   "15 enero 2020", # Spanish
#'   "15 de janeiro de 2020" # Portuguese
#' )
#' fix_date_char(text_months)
#'
#' # Roman numeral months (experimental)
#' roman_dates <- c("15.VII.2023", "3.XII.1999", "1.I.2000")
#' fix_date_char(roman_dates, roman.numeral = TRUE)
#'
#' # Excel serial numbers
#' excel_serials <- c("44197", "44927") # Excel dates
#' fix_date_char(excel_serials, excel = TRUE)
#'
#' # Two-digit years (automatic century detection)
#' two_digit_years <- c("15/03/99", "15/03/25", "15/03/50")
#' fix_date_char(two_digit_years) # 1999, 2025, 1950
#'
#' # MDY format (US style)
#' us_dates <- c("12/25/2023", "07/04/1776", "02/29/2020")
#' fix_date_char(us_dates, format = "mdy")
#'
#' # Incomplete dates with custom imputation
#' incomplete <- c("2023", "March 2022", "June 2021")
#' fix_date_char(incomplete, day.impute = 15, month.impute = 6)
#'
#' @export
fix_date_char <- function(
  dates,
  day.impute = 1,
  month.impute = 7,
  format = "dmy",
  excel = FALSE,
  roman.numeral = FALSE
) {
  # Handle NA input early
  if (length(dates) == 1 && is.na(dates)) {
    return(as.Date(NA))
  }

  # Check non-character input
  if (!is.character(dates)) {
    stop("date should be a character \n")
  }

  # Check day.impute and handle extendr errors
  checkday_result <- checkday(day.impute)
  if (inherits(checkday_result, "extendr_error")) {
    error_msg <- if ("value" %in% names(checkday_result)) {
      checkday_result$value
    } else {
      as.character(checkday_result)
    }
    stop(error_msg, call. = FALSE)
  }

  .checkmonth(month.impute)
  .checkformat(format)

  # Handle NA day.impute by issuing warning
  if (is.na(day.impute)) {
    warning("NA imputed", call. = FALSE)
  }

  # Convert imputation values to integers for Rust
  # Pass -1 as a sentinel value for NULL/NA, which Rust will interpret as None
  day_impute_int <- if (is.na(day.impute) || is.null(day.impute)) -1L else as.integer(day.impute)
  month_impute_int <- if (is.na(month.impute) || is.null(month.impute)) -1L else as.integer(month.impute)

  # Pre-process data to handle NA and empty values
  date_data <- as.character(dates)
  na_indices <- is.na(date_data) | date_data == "" | date_data == "NA"

  # Only process non-NA values through Rust
  if (all(na_indices)) {
    # All values are NA, return all NAs
    fixed_dates <- rep(list(NULL), length(date_data))
  } else {
    # Replace NA values with placeholder for processing
    processed_data <- date_data
    processed_data[na_indices] <- "1999-01-01" # Temporary placeholder

    # Call Rust backend for efficient processing
    fixed_dates <- .Call(
      "wrap__fix_date_column",
      processed_data,
      day_impute_int,
      month_impute_int,
      NULL, # no subjects for character vector processing
      format,
      excel,
      roman.numeral
    )

    # Restore NA values in the result
    if (is.list(fixed_dates)) {
      fixed_dates[na_indices] <- list(NULL)
    } else if (is.vector(fixed_dates)) {
      # Handle case where fixed_dates is a vector
      fixed_dates[na_indices] <- NA
    }
  }

  # Check if the result is an error condition from extendr
  if (inherits(fixed_dates, "extendr_error")) {
    # Extract the error message and throw it as a proper R error
    error_msg <- if ("value" %in% names(fixed_dates)) {
      fixed_dates$value
    } else {
      as.character(fixed_dates)
    }
    stop(error_msg, call. = FALSE)
  }

  # Convert to Date class, handling NULL values from Rust
  result <- as.Date(sapply(fixed_dates, function(x) if (is.null(x)) NA_character_ else x))

  # Remove names to match expected output
  names(result) <- NULL

  result
}

test_that("fix_date_char works for a series of malformed dates", {
  dates <- c(
    "02 03 2021" = "2021-03-02",
    "15/07/11" = "2011-07-15",
    "15/07/84" = "1984-07-15",
    "2015 11 11" = "2015-11-11",
    "05/2015" = "2015-05-01",
    "2020-01" = "2020-01-01",
    "05/89" = "1989-05-01",
    "02/14" = "2014-02-01",
    "1994" = "1994-07-01"
  )

  fixed <- fix_date_char(names(dates))

  expect_equal(fixed, as.Date(unname(dates)))
})

test_that("fix_date returns NA if NA is given", {
  bad.date <- NA
  fixed.date <- fix_date_char(bad.date)
  expect_equal(fixed.date, as.Date(NA))
})

test_that("error if date given isn't character", {
  bad.date <- 15
  expect_error(fix_date_char(bad.date), "date should be a character \n")
})

test_that("error if date given has over 4 digits for year", {
  bad.date <- "202312-12-2"
  expect_error(fix_date_char(bad.date), "unable to tidy a date")
})

test_that("Test NA imputed when provided", {
  bad.date <- "1990-5"
  expect_warning(
    date.guess <- fix_date_char(bad.date, day.impute = NA),
    "NA imputed"
  )
  expect_equal(date.guess, as.Date(NA))
})

test_that("fix_date works for mdy format", {
  bad.date <- "07/15/11"
  fixed.date <- fix_date_char(bad.date, format = "mdy")
  expect_equal(fixed.date, as.Date("2011-07-15"))
})

test_that("unexpected format raises error", {
  bad.date <- "07/15/11"
  expect_error(
    fix_date_char(bad.date, format = "ydm"),
    "format should be either 'dmy' or 'mdy' \n"
  )
})

test_that("'de' and 'del' is parsed", {
  expect_equal(fix_date_char("20 de abril de 1994"), as.Date("1994-04-20"))
  expect_equal(fix_date_char("06 de enero del 2008"), as.Date("2008-01-06"))
})

test_that("German format days are supported", {
  expect_equal(fix_date_char("29.08.1992"), as.Date("1992-08-29"))
})

test_that("nunber. + month name (whitespace) + year", {
  expect_equal(fix_date_char("3. Oktober 1990"), as.Date("1990-10-03"))
})

test_that("Month-first dates where the month is given by name", {
  expect_equal(fix_date_char("July 4th, 1776"), as.Date("1776-07-04"))
})

test_that("Non-excel numeric date is parsed correctly", {
  expect_equal(fix_date_char("19374"), as.Date("2023-01-17"))
})

test_that("Excel numeric date is parsed correctly", {
  expect_equal(fix_date_char("41035", excel = TRUE), as.Date("2012-05-06"))
})

test_that("Allow single digit day with double digit year", {
  expect_equal(fix_date_char("03/10/90"), fix_date_char("3/10/90"))
})

test_that("if day > max for month, lower day to max", {
  expect_equal(fix_date_char("2023-02-30"), as.Date("2023-02-28"))
  expect_equal(fix_date_char("2024-02-30"), as.Date("2024-02-29"))
})

test_that("Error if day of month > 31 or < 1", {
  expect_error(fix_date_char("32-01-2023"), "Day not in expected range\n")
})


test_that("Roman conversion works as expected", {
  expect_equal(
    fix_date_char("2023-II-15", roman.numeral = TRUE),
    as.Date("2023-02-15")
  )
})

test_that("NA values in vectors are preserved correctly", {
  # Test vector with mixed NA, empty string, and "NA" string
  test_vector <- c("2023-01-01", NA, "2024-01-01", "", "NA")
  result <- fix_date_char(test_vector)

  # Check that valid dates are parsed correctly
  expect_equal(result[1], as.Date("2023-01-01"))
  expect_equal(result[3], as.Date("2024-01-01"))

  # Check that all types of NA are preserved as NA
  expect_true(is.na(result[2])) # NA
  expect_true(is.na(result[4])) # empty string
  expect_true(is.na(result[5])) # "NA" string

  # Ensure we didn't get the placeholder date (1999-01-01)
  expect_false(any(result == as.Date("1999-01-01"), na.rm = TRUE))
})

test_that("Vector with all NA values returns all NA", {
  test_vector <- c(NA, "", "NA")
  result <- fix_date_char(test_vector)

  # All should be NA
  expect_true(all(is.na(result)))
  expect_equal(length(result), 3)

  # Ensure we didn't get the placeholder date
  expect_false(any(result == as.Date("1999-01-01"), na.rm = TRUE))
})

test_that("? is treated as NA", {
  expect_equal(fix_date_char("?"), as.Date(NA))
  expect_equal(
    fix_date_char(c("01/01/2020", "?", "02/02/2021")),
    as.Date(c("2020-01-01", NA, "2021-02-02"))
  )
})

test_that("error if date has more than two separators", {
  expect_error(fix_date_char("01/02/03/2000"), "unable to tidy a date")
  expect_error(fix_date_char("01-02-03-2000"), "unable to tidy a date")
  expect_error(fix_date_char("01 02 03 2000"), "unable to tidy a date")
})

test_that("parse error includes element index and date", {
  expect_error(
    fix_date_char("202312-12-2"),
    "for subject 1 \\(date: 202312\\-12\\-2\\)"
  )
})

test_that("parse error reports correct index when error is not in first element", {
  expect_error(
    fix_date_char(c("01/01/2020", "02/02/2021", "202312-12-2")),
    "for subject 3 \\(date: 202312\\-12\\-2\\)"
  )
})

test_that("Legitimate 1999-01-01 dates are preserved correctly", {
  # Test vector with legitimate 1999-01-01 dates mixed with NA and other dates
  test_vector <- c("1999-01-01", NA, "2023-01-01", "", "January 1, 1999", "NA")
  result <- fix_date_char(test_vector)

  # Check that legitimate 1999-01-01 dates are preserved
  expect_equal(result[1], as.Date("1999-01-01"))
  expect_equal(result[5], as.Date("1999-01-01")) # "January 1, 1999"
  expect_equal(result[3], as.Date("2023-01-01"))

  # Check that NA values are still NA
  expect_true(is.na(result[2])) # NA
  expect_true(is.na(result[4])) # empty string
  expect_true(is.na(result[6])) # "NA" string

  # Count legitimate 1999-01-01 dates
  count_1999 <- sum(!is.na(result) & result == as.Date("1999-01-01"))
  expect_equal(count_1999, 2)
})

test_that("check Roman numeral dates work", {
  messy.dates <- c("01 ii 2021", "2023-iv-02", "13/ix/1975")
  expect_equal(
    fix_date_char(messy.dates, roman.numeral = TRUE),
    as.Date(c("2021-02-01", "2023-04-02", "1975-09-13"))
  )
})

# Generated by extendr: Do not edit by hand

# nolint start

#
# This file was created with the following call:
#   .Call("wrap__make_datefixR_wrappers", use_symbols = TRUE, package_name = "datefixR")

#' @usage NULL
#' @useDynLib datefixR, .registration = TRUE
NULL

#' Validate day imputation value range
#' @noRd
checkday <- function(day_impute) .Call(wrap__checkday, day_impute)

#' Main date fixing function - Rust implementation of .fix_date
#' @noRd
fix_date <- function(date, day_impute, month_impute, subject, format, excel, roman_numeral) .Call(wrap__fix_date, date, day_impute, month_impute, subject, format, excel, roman_numeral)

#' Analyze and fix date strings in a whole column of a DataFrame
#' @noRd
fix_date_column <- function(dates, day_impute, month_impute, subjects, format, excel, roman_numeral) .Call(wrap__fix_date_column, dates, day_impute, month_impute, subjects, format, excel, roman_numeral)


# nolint end

#' @noRd
.checkmonth <- function(month.impute) {
  if (!is.na(month.impute) && !is.null(month.impute)) {
    if (month.impute < 1 || month.impute > 12) {
      stop("month.impute should be an integer between 1 and 12\n")
    }
    if (!(month.impute %% 1 == 0)) {
      stop("month.impute should be an integer\n")
    }
  }
  return()
}


#' @noRd
.checkformat <- function(format) {
  if (!(format %in% c("dmy", "mdy"))) {
    stop("format should be either 'dmy' or 'mdy' \n")
  }
}

#' Example dataset of dates in different formats
#'
#' A toy dataset to use with datefixR functions.
#'
#' @format A data frame with 5 rows and 3 variables:
#' \describe{
#'   \item{id}{Row ID (numeric).}
#'   \item{some.dates}{Dates in different formats (character).}
#'   \item{some.more.dates}{Additional dates in different formats (character).}
#' }
"exampledates"

library(testthat)
if (Sys.info()[["sysname"]] == "Darwin") {
  test_that("Warning if multibyte support is not detected", {
    withr::with_locale(new = c("LC_CTYPE" = "en_US.US-ASCII"), code = {
      expect_message(
        library(datefixR),
        paste0(
          "The current locale does not support multibyte characters. ",
          "You may run into difficulties if any months are given as ",
          "non-English language names. \n"
        )
      )
    })
  })
}

---
title: "Localization with datefixR"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Internationalization with datefixR}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(datefixR)
```

## Introduction

One of the most powerful features of `datefixR` is its comprehensive
localization support. The package can automatically recognize and parse
dates in multiple languages without requiring users to specify the language
beforehand. This makes it particularly valuable for processing datasets
collected across different countries or containing mixed-language date entries.

This vignette provides detailed examples and best practices for working with
international date formats in `datefixR`.

## Supported Languages and Formats

`datefixR` currently supports date parsing in **7 languages**, with both
abbreviated and full month names:

### Fully Supported Languages

- **English**: Default language, supports both US (MDY) and international (DMY) formats
- **French (Français)**: Comprehensive support including cultural formatting patterns
- **German (Deutsch)**: Full month names and common German date conventions
- **Spanish (Español)**: Including regional variations and prepositions
- **Portuguese**: Brazilian and European Portuguese month names
- **Russian (Русский)**: Cyrillic month names and formatting
- **Indonesian (Bahasa Indonesia)**: Indonesian month names\

### Language Detection Features

- **Automatic Recognition**: No need to specify language in advance
- **Mixed Language Support**: Process datasets with multiple languages simultaneously
- **Cultural Awareness**: Recognizes language-specific formatting conventions
- **Flexible Parsing**: Handles variations in capitalization and spacing
- **User messages**: User-facing messages have been translated for the above 
  languages in addition to Czech and Slovak. 

## Language-Specific Examples

### English

English dates support various formats including ordinal numbers and different separator styles:

```{r}
# Standard formats
fix_date_char("15 January 2020")
fix_date_char("July 4th, 1776")
fix_date_char("December 25th, 2023")

# Different separators
english_dates <- c( # US format with slashes
  "15-Jan-2020", # Day-month-year with hyphens
  "2020.01.15", # Year-first with dots
  "15th January 2020" # Full format with ordinal
)
fix_date_char(english_dates)
```

### French (Français)

French dates often include articles ("le", "du") and prepositions ("de"):

```{r}
# Various French formats
french_dates <- c(
  "15 janvier 2020", # Standard format
  "le 3 mars 2013", # With article
  "1er avril 2022", # First of the month
  "25 déc. 2023", # Abbreviated December
  "3 mai 1968" # Historical date
)
fix_date_char(french_dates)

# Mixed separators in French
french_mixed <- c(
  "15-janvier-2020",
  "03/mars/2013",
  "2020.05.15"
)
fix_date_char(french_mixed)
```

### German (Deutsch)

German dates commonly use dots as separators and may include ordinal indicators:

```{r}
# German date formats
german_dates <- c(
  "15. Januar 2020", # With ordinal dot
  "3. Dezember 1999", # December
  "1. Mai 2023", # May Day
  "24. Dez 2023", # Abbreviated December
  "15 Jan 2020" # Without ordinal
)
fix_date_char(german_dates)

# German with different separators
german_separators <- c(
  "15.01.2020",
  "15-Januar-2020",
  "15/01/20"
)
fix_date_char(german_separators)
```

### Spanish (Español)

Spanish dates frequently include prepositions like "de" and "del":

```{r}
# Spanish date variations
spanish_dates <- c(
  "15 de enero de 2020", # Full format with prepositions
  "7 de septiembre del 2014", # With "del" contraction
  "1 ene 2023", # Abbreviated January
  "25 dic 2023", # Abbreviated December
  "15 enero 2020" # Without prepositions
)
fix_date_char(spanish_dates)

# Regional variations
spanish_regional <- c(
  "15-ene-2020",
  "15/enero/2020",
  "2020-01-15"
)
fix_date_char(spanish_regional)
```

### Portuguese

Portuguese dates, like Spanish, use prepositions and have both Brazilian and European variations:

```{r}
# Portuguese date formats
portuguese_dates <- c(
  "15 de janeiro de 2020", # Full format
  "3 de dezembro de 1999", # December
  "1º jan 2023", # First with ordinal
  "25 dez 2023", # Abbreviated
  "15 janeiro 2020" # Without prepositions
)
fix_date_char(portuguese_dates)
```

### Russian (Русский)

Russian uses Cyrillic script for month names:

```{r}
# Russian date examples
russian_dates <- c(
  "15 января 2020", # January
  "3 декабря 1999", # December
  "1 мая 2023", # May
  "25 дек 2023" # Abbreviated December
)
fix_date_char(russian_dates)
```

### Indonesian (Bahasa Indonesia)

```{r}
# Indonesian date examples
indonesian_dates <- c(
  "15 Januari 2020", # January
  "3 Desember 1999", # December
  "1 Mei 2023" # May
)
fix_date_char(indonesian_dates)
```

## Handling Mixed Languages

The package automatically detects the language of month names without requiring
explicit specification:

```{r}
mixed_dates <- c(
  "15 January 2020", # English
  "15 janvier 2020", # French
  "15 Januar 2020", # German
  "15 enero 2020", # Spanish
  "15 de janeiro de 2020" # Portuguese
)
fix_date_char(mixed_dates)
```

## Format Detection

`datefixR` intelligently handles different date formats and separators across languages:

```{r}
international_formats <- c(
  "02/05/92", # US/UK ambiguous
  "2020-mai-01", # French with hyphen
  "1996.05.01", # German with dots
  "02 04 96" # Space separator
)
fix_date_char(international_formats)
```

## Roman Numeral Support (Experimental)

The package includes experimental support for Roman numeral months:

```{r}
roman_dates <- c("15.VII.2023", "3.XII.1999", "1.I.2000")
fix_date_char(roman_dates, roman.numeral = TRUE)
```

## Best Practices

1. **No Language Specification Required**: The package automatically detects the language
2. **Mixed Language Support**: You can process datasets with multiple languages
3. **Format Flexibility**: Various separators and formats are supported
4. **Consistent Output**: All dates are converted to standardized R Date class

## Adding New Languages

If you need/want support for additional languages, please consider contributing
to the package or filing an issue on GitHub.

---
title: "datefixR: Comprehensive Date Standardization Guide"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{datefixR: Comprehensive Date Standardization Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  warning = FALSE,
  message = FALSE
)
```

```{r setup}
library(datefixR)
```

# Introduction

**datefixR** is a comprehensive R package designed to automatically standardize messy date data into consistent, machine-readable formats. Whether you're dealing with mixed date formats from web forms, international datasets, or legacy systems, `datefixR` intelligently parses diverse date representations and converts them to R's standard `Date` class.

## Key Features

- **Smart parsing**: Handles mixed date formats, separators, and representations in a single dataset
- **Multilingual support**: Recognizes dates in English, French, German, Spanish, Portuguese, Russian, Czech, Slovak, and Indonesian
- **Missing data imputation**: User-controlled behavior for incomplete dates (missing days/months)
- **Detailed error reporting**: Identifies exactly which dates couldn't be parsed and why
- **Excel compatibility**: Supports both R and Excel numeric date representations
- **Roman numeral support**: Experimental support for Roman numeral months

## Core Functions

The package consists of three main user-accessible functions:

- **`fix_date_char()`**: Converts character vectors of dates
- **`fix_date_df()`**: Processes date columns in data frames or tibbles
- **`fix_date_app()`**: Interactive Shiny application for date cleaning (see dedicated vignette)

## When to Use datefixR

This package is particularly valuable when:

- Processing dates from free-text web form entries
- Handling international datasets with mixed date formats
- Cleaning legacy data with inconsistent date representations
- Working with Excel imports containing mixed text and numeric dates
- Validating and standardizing date data quality

# Core Workflow

### Date standardization

Firstly, we will demonstrate date standardization without imputation. We
consider a data frame with two columns of dates in differing formats
with no missing data. 

```{r, echo = TRUE, results='asis'}
bad.dates <- data.frame(
  id = seq(5),
  some.dates = c(
    "02/05/92",
    "01-04-2020",
    "1996/05/01",
    "2020-05-01",
    "02-04-96"
  ),
  some.more.dates = c(
    "01 03 2015",
    "2nd January 2010",
    "01/05/1990",
    "03-Dec-2012",
    "02 April 2020"
  )
)
knitr::kable(bad.dates)
```

`fix_date_df()` requires two arguments, `df`, a data frame (or tibble) object
and `col.names`, a character vector containing the names of columns to be
standardized. By default, the first column of the data frame is assumed to
contain row IDs. These IDs are used if a warning or error is raised to assist
the user with locating the source of the error. The ID column can also be
manually provided via the `id` argument. 

The output from this function is a data frame or tibble (dependent on the
object type of the the first argument, `df`) with the selected date
columns now standardized and belonging to the `Date` class. 

```{r, echo = TRUE, results='asis'}
fixed.dates <- fix_date_df(
  bad.dates,
  c("some.dates", "some.more.dates")
)
knitr::kable(fixed.dates)
```

`datefixR` can handle many different formats including -, /, ., or white space
separation,  year-first or day-first, and month supplied as a number, an
abbreviation or full length name.

`fix_date_char()` is similar to `fix_date_df()` but only applies to a single
character object. 

```{r}
fix_date_char("01 02 2014")
```

# Advanced Topics

## Localization

`datefixR` currently supports dates being provided in English, Français
(French), Deutsch (German), español (Spanish), and Русский (Russian) by
recognizing both names of months in these languages and formatting customs.
Expected languages do not need to be specified and can be provided just like any
other date to be standardized.

 
```{r}
fix_date_char("7 de septiembre del 2014")
```

Functions in `datefixR` assume day-first instead of month-first when day, month,
and year are all given numerically (unless year is given first). However, this
behavior can be modified by passing `format = "mdy"` to function calls.  

```{r}
fix_date_char("01 02 2014", format = "mdy")
```

If the month is given by name, then `datefixR` will automatically detect the
correct format without the `format` argument needing to be specified by the user. 

```{r}
fix_date_char("July 4th, 1776")
```


## Date and Month Imputation

By default, `datefixR` imputes missing months as July, and missing days of the
month as the first day.  As such, "1992" converts to 

```{r}
fix_date_char("1992")
```

The argument for defaulting to July is 1st-2nd July is halfway through the year
(on a non leap year). Therefore, assuming the year supplied is indeed correct,
the imputation has a maximum potential error of 6 months from the true date.
However, this behavior can be changed by supplying the `day.impute` and 
`month.impute` arguments with an integer corresponding to the desired day and
month. For example, `day.impute = 1` and `month.impute = 1`  results in the
first day of January being imputed instead which is often a more common
imputation for missing date data. 

```{r}
fix_date_char("1992", day.impute = 1, month.impute = 1)
```

The imputation mechanism can also be modified to impute `NA` if a month or day
is missing by setting `day.impute` or `month.impute` to `NA`. This will
also result in a warning being raised.

```{r}
fix_date_char("1992", month.impute = NA)
```

Finally, imputation can be prevented by setting `day.impute` or `month.impute`
to `NULL`. This will result in an error being raised if the day or month
are missing respectively. 

```{r, eval = FALSE}
fix_date_char("1992", month.impute = NULL)
# ERROR
```

`day.impute` and `month.impute` can also be passed to `fix_date_df()` for
similar functionality. 

```{r}
example.df <- data.frame(
  id = seq(1, 3),
  some.dates = c("2014", "April 1990", "Mar 19")
)
fix_date_df(example.df, "some.dates", day.impute = 1, month.impute = 1)
```

## Converting Numeric Dates

By default, if a date is given numerically (I.E no separators such as "/", "-",
or white space) and is more than four character long, then this date is assumed
to have been converted from `R`'s numeric date format. If a `Date` object is
converted to a `numeric` object in R, the assigned value becomes the number of
days from `1970-01-01`. Note that the date must still be converted to a
`character` object before being passed to a `datefixR` function.

```{r}
date <- as.numeric(as.Date("2023-01-17"))
print(date)
fix_date_char(as.character(date))
```

However if a date is converted to a numeric date in Excel, the number of days
is instead counted from `1900-01-01`. If a user expects that dates to have been
converted to numerical dates in Excel, then `excel = TRUE` can be passed to a
`datefixR` function to rectify this. 

```{r}
fix_date_char("44941", excel = TRUE)
```

## Roman Numeral Months [![Experimental](https://lifecycle.r-lib.org/articles/figures/lifecycle-experimental.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)

Oracle Database can use Roman numerals to format months and this custom is also
used in some biological contexts. If dates in need of parsing are in this format,
`roman.numeral = TRUE` can be passed to `fix_date_char()` or `fix_date_df()`.
This implementation is currently experimental and is not guaranteed to work
alongside other date formats. 

```{r}
fix_date_char("12/IV/2019", roman.numeral = TRUE)
```

# Error & Edge-Case Handling

`datefixR` provides detailed error messages when it encounters dates which
cannot be parsed.
These errors often guide you to correct format issues or identify unsupported cases.

### Common Error Examples

- **Unsupported Date Parts**: If month names or formats aren't recognized.
- **Invalid Date Formats**: Entering purely numeric values with unsupported separators.

```{r}
tryCatch(
  {
    fix_date_char("99-99-9999")
  },
  error = function(e) {
    cat("Error:", e$message, "\n")
  }
)
```

# FAQ

## Understanding Year Parsing Logic

### Two-Digit vs Four-Digit Years

`datefixR` implements intelligent two-digit year expansion using a sliding window approach. The algorithm examines the first digit of a two-digit year and compares it to the third digit of the current year.

**Algorithm Details:**
- If the first digit ≤ current year's third digit: prefix with "20"
- If the first digit > current year's third digit: prefix with "19"

```{r eval=FALSE}
# Current year: 2025 (third digit is 2)
# Years 00-25 become 2000-2025
fix_date_char("01/01/05") # → 2005-01-01
fix_date_char("01/01/24") # → 2024-01-01

# Years 26-99 become 1926-1999
fix_date_char("06/15/92") # → 1992-06-15
fix_date_char("03/10/80") # → 1980-03-10
```

**Edge Case Behavior:**
As time progresses, this window shifts naturally. In 2030, years 00-30 will map to 2000-2030, while 31-99 map to 1931-1999.

```{r}
# Demonstrating current behavior (as of 2025)
samples <- c("01/01/20", "01/01/24", "01/01/23", "01/01/50", "01/01/99")
for (date in samples) {
  result <- fix_date_char(date)
  cat(sprintf("%s → %s\n", date, result))
}
```

## Automatic YMD Detection Logic

`datefixR` uses a sophisticated hierarchy to determine date component order:

### 1. Year-First Detection
If the first component is 4 digits, assumes `YYYY-MM-DD` format:
```{r}
fix_date_char("2023/12/25") # Automatically detects year-first
fix_date_char("1995-04-15") # Year-first with different separator
```

### 2. Month Name Detection
If the first component is a month name, switches to `month-day-year` format:
```{r}
fix_date_char("January 15, 2023") # Month name → MDY
fix_date_char("Mar 5 1992") # Abbreviated month → MDY
fix_date_char("abril 20 2020") # Spanish month → MDY
```

### 3. Numeric Component Defaults
For purely numeric dates, defaults to `day-month-year` unless overridden:
```{r}
# Default behavior (DMY)
fix_date_char("15/03/2023") # → 2023-03-15 (day/month/year)

# Override with format parameter
# fix_date_char("15/03/2023", format = "mdy") # → Invalid (month 15)
fix_date_char("03/15/2023", format = "mdy") # → 2023-03-15 (month/day/year)
```

### 4. Handling Ambiguous Cases

`datefixR` cannot resolve truly ambiguous dates without explicit format specification:

```{r}
# Ambiguous: could be March 5th or May 3rd
fix_date_char("03/05/2023") # → 2023-05-03 (assumes DMY)
fix_date_char("03/05/2023", format = "mdy") # → 2023-03-05 (forces MDY)

# Unambiguous: day > 12 forces correct interpretation
fix_date_char("15/03/2023") # → 2023-03-15 (only valid as DMY)
fix_date_char("03/15/2023", format = "mdy") # → 2023-03-15 (only valid as MDY)
```

### 5. Format Detection Examples

Here's how different inputs trigger specific detection logic:

```{r}
# Year-first detection (4-digit first component)
test_dates_ymd <- c("2023/01/15", "1999-12-31", "2020.06.30")
for (date in test_dates_ymd) {
  cat(sprintf("%s → %s (YMD detected)\n", date, fix_date_char(date)))
}

# Month-name detection (text month triggers MDY)
test_dates_mdy <- c("March 15, 2023", "Dec 25 2020", "Jan 1st 2000")
for (date in test_dates_mdy) {
  cat(sprintf("%s → %s (MDY detected)\n", date, fix_date_char(date)))
}

# Default numeric (assumes DMY)
test_dates_dmy <- c("15/03/2023", "01-12-1999", "25.12.2020")
for (date in test_dates_dmy) {
  cat(sprintf("%s → %s (DMY default)\n", date, fix_date_char(date)))
}
```

## Overriding Automatic Detection

### Explicit Format Specification

When automatic detection fails or produces unwanted results, use the `format` parameter:

```{r}
# Force MDY interpretation
fix_date_char("01/02/2023", format = "mdy") # → 2023-01-02 (Jan 2nd)
fix_date_char("01/02/2023", format = "dmy") # → 2023-02-01 (Feb 1st)

# Useful for consistently formatted datasets
dates_usa <- c("01/15/2023", "03/22/2023", "12/01/2023")
lapply(dates_usa, function(x) fix_date_char(x, format = "mdy"))
```

### Data Frame Processing

The same logic applies to `fix_date_df()` with consistent format specification:

```{r}
usa_dates <- data.frame(
  id = 1:3,
  event_date = c("01/15/2023", "03/22/2023", "12/01/2023")
)

# Apply consistent MDY format
fixed_usa <- fix_date_df(usa_dates, "event_date", format = "mdy")
knitr::kable(fixed_usa)
```

## Citation

If you use this package in your research, please consider citing `datefixR`.
An up-to-date citation can be obtained by running

```{r}
citation("datefixR")
```

---
title: "Using the datefixR Shiny App"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using the datefixR Shiny App}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(datefixR)
```

## Introduction

The `datefixR` package provides a user-friendly Shiny app that allows users to standardize messy date data using a graphical user interface (GUI). This is particularly useful for researchers, data analysts, and anyone working with datasets containing inconsistently formatted dates who prefer not to use R code directly.

The app supports the same powerful date parsing capabilities as the core `datefixR` functions, including:

- Multiple date formats and separators
- International month names in 9 languages
- Custom imputation strategies for missing date components
- Excel and CSV file processing
- Batch processing of multiple date columns

## Installation and Setup

### Prerequisites

The Shiny app requires additional dependencies that are not installed automatically with `datefixR`. This design choice allows the core package to be installed on secure systems where these packages might not be permitted.

Required dependencies:
- `DT` - for interactive data tables
- `shiny` - for the web application framework
- `readxl` - for reading Excel files
- `htmltools` - for HTML generation

### Launching the App

To start the app, simply run:

```{r eval=FALSE}
fix_date_app()
```

If any required dependencies are missing, the app will detect this and offer to install them automatically.

### Theme Options

The app supports two visual themes:

```{r eval=FALSE}
# Default datefixR theme (recommended)
fix_date_app(theme = "datefixR")

# Standard Shiny theme
fix_date_app(theme = "none")
```

## Step-by-Step Usage Guide

### 1. File Upload

- Click the **Browse** button in the left sidebar
- Select either a CSV (.csv) or Excel (.xlsx) file
- The file will be automatically uploaded and processed
- Click **Refresh** to display the uploaded data

### 2. Column Selection

After uploading your file:

- Review the data in the **Results** tab
- In the left sidebar, check boxes will appear for each column
- Select the columns containing date data that need to be standardized
- Multiple columns can be selected simultaneously

### 3. Imputation Settings

Configure how missing date components should be handled:

#### Day of Month Imputation
- **Options**: 1-28, or NA
- **Default**: 1 (first day of month)
- **Purpose**: What day to use when only month/year are provided

#### Month Imputation
- **Options**: 1-12, or NA
- **Default**: 7 (July)
- **Purpose**: What month to use when only year is provided

#### Format Assumption
- **Options**: "dmy" (day-month-year) or "mdy" (month-day-year)
- **Default**: "dmy"
- **Purpose**: How to interpret ambiguous numeric dates like "01/02/2023"

### 4. Processing and Review

- Click **Refresh** after selecting columns and setting preferences
- The processed data will appear in the **Results** tab
- Review the standardized dates to ensure they meet your expectations
- All date columns will now be in YYYY-MM-DD format

### 5. Download Results

- Click the **Download** button to save your processed data
- The file will be saved as "fixed.csv"
- All selected date columns will be standardized in the output

## Advanced Features

### File Format Support

The app automatically detects and handles:

- **CSV files**: Comma-separated values with automatic encoding detection
- **Excel files**: Both .xlsx and .xls formats, reads the first worksheet

### Error Handling

If the app encounters problematic dates:

- Error messages will appear in the R console
- The problematic row and date value will be identified
- Processing will stop, allowing you to review and correct the data

### Data Privacy

**Important Security Note**: When using online hosting platforms (like shinyapps.io), your uploaded files are temporarily stored on the hosting platform's servers. While no data should be stored persistently, use discretion with sensitive data. For maximum security, run the app locally.

## Example Workflow

Here's a complete example of using the app:

1. **Prepare your data**: Create a CSV file with messy dates
   ```
   id,event_date,follow_up
   1,"02/03/21","April 2021"
   2,"15-Dec-2020","2021"
   3,"2020/05/01","May 15 2021"
   ```

2. **Launch and configure**:
   ```{r eval=FALSE}
fix_date_app()
   ```

3. **Upload and process**:
   - Upload your CSV file
   - Select "event_date" and "follow_up" columns
   - Set day imputation to 15, month imputation to 6
   - Click Refresh

4. **Download results**: Clean, standardized date data ready for analysis

## Troubleshooting

### Common Issues

**App won't start**:
- Ensure all dependencies are installed
- Try running `install.packages(c("DT", "shiny", "readxl", "htmltools"))`

**File won't upload**:
- Check file format (only .csv and .xlsx supported)
- Ensure file size is reasonable (< 100MB recommended)
- Verify file isn't corrupted

**Dates not parsing correctly**:
- Review your format assumption (dmy vs mdy)
- Check for unusual date formats in your data
- Consider pre-cleaning obviously problematic entries

**Download not working**:
- Ensure you've selected at least one date column
- Try refreshing the processed data first
- Check browser download settings

## Performance Considerations

- **File Size**: The app handles files up to several thousand rows efficiently
- **Processing Time**: Complex date parsing may take a few seconds for large datasets
- **Memory Usage**: Keep file sizes reasonable (< 100MB) for optimal performance

For the latest updates and to report issues, visit the [datefixR GitHub repository](https://github.com/ropensci/datefixR).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fix_date_df.R
\name{fix_date_df}
\alias{fix_date_df}
\title{Clean up messy date columns}
\usage{
fix_date_df(
  df,
  col.names,
  day.impute = 1,
  month.impute = 7,
  id = NULL,
  format = "dmy",
  excel = FALSE,
  roman.numeral = FALSE,
  cores = getOption("Ncpus", 1)
)
}
\arguments{
\item{df}{A \code{dataframe} or \code{tibble} object containing messy date
column(s).}

\item{col.names}{Character vector specifying column names of date data to be
cleaned.}

\item{day.impute}{Integer between 1 and 31, or NA, or NULL. Day of the month
to be imputed when missing. Defaults to 1. If \code{day.impute} is greater
than the number of days in a given month, the last day of that month will
be imputed (accounting for leap years). If \code{day.impute = NA}, then
\code{NA} will be imputed for the entire date and a warning will be raised.
If \code{day.impute = NULL}, the function will fail with an error when day
is missing.}

\item{month.impute}{Integer between 1 and 12, or NA, or NULL. Month to be
imputed when missing. Defaults to 7 (July). If \code{month.impute = NA},
then \code{NA} will be imputed for the entire date and a warning will be
raised.
If \code{month.impute = NULL}, the function will fail with an error when
month is missing.}

\item{id}{Optional parameter specifying the name of the column containing
row IDs. Defaults to using the first column for IDs.}

\item{format}{Character string specifying date interpretation preference.
Either \code{"dmy"} (day-month-year, default) or \code{"mdy"}
(month-day-year, US format). This setting only affects ambiguous numeric
dates like "01/02/2023". When month names are present or year appears
first, the format is auto-detected regardless of this parameter. Note that
unambiguous dates (e.g., "25/12/2023") are parsed correctly regardless of
the format setting.}

\item{excel}{Logical: Assumes \code{FALSE} by default. If \code{TRUE}, treats
numeric-only dates with more than four digits as Excel serial dates with
1900-01-01 origin, correcting for known Excel date discrepancies.}

\item{roman.numeral}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}} Logical: Defaults
to \code{FALSE}. When \code{TRUE}, attempts to interpret Roman numeral
month indications within datasets. This feature may not handle all cases
correctly.}

\item{cores}{Integer: Number of CPU cores to use for parallel processing.
Defaults to \code{getOption("Ncpus", 1)}. When \code{cores > 1}, processes
multiple date columns in parallel using the \code{future} framework.
Requires the \code{future} and \code{future.apply} packages to be installed.
The actual number of workers used will be the minimum of \code{cores} and
the number of columns to process.}
}
\value{
A revised \code{dataframe} or \code{tibble} structure, maintaining
input type. Date columns will be formatted with \code{Date} class and
display as \code{yyyy-mm-dd}.
}
\description{
Tidies a \code{dataframe} or \code{tibble} object with date
columns entered via a free-text interface, addressing non-standardized
formats. Supports diverse separators including /, -, ., and spaces. Handles
all-numeric, abbreviated, or full-length month names in languages such as
English, French, German, Spanish,  Portuguese, Russian, Czech, Slovak, and
Indonesian. Imputes missing day data by  default, with flexibility for custom
imputation strategies.
}
\details{
This function processes messy date data by:
\itemize{
\item{Supporting mixed format data entries}
\item{Recognizing multilingual month names and Roman numeral inputs}
\item{Interpreting Excel-style serial date numbers if specified}
\item{Providing warnings and controls for missing day/month imputation}
}
For further details and advanced usage, refer to the vignette via
\code{browseVignettes("datefixR")} or visit the online documentation at
\url{https://docs.ropensci.org/datefixR/}.
}
\examples{
# Basic cleanup
data(exampledates)
fix_date_df(exampledates, c("some.dates", "some.more.dates"))

# Usage with metadata
messy_dates_df <- data.frame(
  id = seq(1, 3),
  dates = c("1992", "April 1990", "Mar 19")
)
fix_date_df(messy_dates_df, "dates", day.impute = 15, month.impute = 12)

# Diverse format normalization
df_formats <- data.frame(
  mixed.dates = c("02/05/92", "2020-may-01", "1996.05.01", "October 2022"),
  european.dates = c("22.07.1977", "05.06.2023")
)
fix_date_df(df_formats, c("mixed.dates", "european.dates"))

# Excel serial examples
serial_df <- data.frame(serial.dates = c("44197", "44927"))
fix_date_df(serial_df, "serial.dates", excel = TRUE)

# Handling Roman numerals
roman_df <- data.frame(roman.dates = c("15.I.2023", "03.XII.2019"))
fix_date_df(roman_df, "roman.dates", roman.numeral = TRUE)

# Parallel processing (requires 'future' and 'future.apply' packages)
\dontrun{
large_df <- data.frame(
  dates1 = c("01/02/2020", "15/03/2021", "22/12/2019"),
  dates2 = c("2020-01-01", "March 2021", "Dec 2019"),
  dates3 = c("01.01.20", "15.03.21", "22.12.19")
)
# Use 4 cores for parallel processing
fix_date_df(large_df, c("dates1", "dates2", "dates3"), cores = 4)

# Use all available cores (respects getOption("Ncpus"))
options(Ncpus = parallel::detectCores())
fix_date_df(large_df, c("dates1", "dates2", "dates3"))
}
}
\seealso{
\code{\link{fix_date_char}} for similar functionality on character vectors.

For comprehensive examples and usage practices, consult:
\itemize{
\item{Vignette: \code{browseVignettes("datefixR")}}
\item{Documentation: \url{https://docs.ropensci.org/datefixR/articles/datefixR.html}}
\item{README Overview: \url{https://docs.ropensci.org/datefixR/}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exampledates}
\alias{exampledates}
\title{Example dataset of dates in different formats}
\format{
A data frame with 5 rows and 3 variables:
\describe{
\item{id}{Row ID (numeric).}
\item{some.dates}{Dates in different formats (character).}
\item{some.more.dates}{Additional dates in different formats (character).}
}
}
\usage{
exampledates
}
\description{
A toy dataset to use with datefixR functions.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shinyapp.R
\name{fix_date_app}
\alias{fix_date_app}
\title{Shiny application standardizing date data in csv or excel files}
\usage{
fix_date_app(theme = "datefixR")
}
\arguments{
\item{theme}{Color theme for shiny app. Either \code{"datefixR"}
(\code{datefixR} colors) or \code{"none"}(default shiny app styling).}
}
\value{
A shiny app.
}
\description{
A shiny application which allows users to standardize dates
using a graphical user interface (GUI). Most features of \code{datefixR}
are supported including imputing missing date data. Data can be provided as
CSV (comma-separated value) or XLSX (Excel) files. Processed datasets can
be downloaded as CSV files. Please note, the dependencies for this app
(\code{DT}, \code{htmltools}, \code{readxl}, and \code{shiny}) are not
installed alongside \code{datefixR}. This allows \code{datefixR} to be
installed on secure systems where these packages may not be allowed. If one
of these dependencies is not installed on the system when this function is
called, then the user will be given the option of installing them.
}
\examples{
\dontrun{
fix_date_app()
}
}
\seealso{
The \code{\link[shiny]{shiny}} package.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fix_date_char.R
\name{fix_date_char}
\alias{fix_date_char}
\title{Convert non-standardized dates to \R{}'s \code{Date} class}
\usage{
fix_date_char(
  dates,
  day.impute = 1,
  month.impute = 7,
  format = "dmy",
  excel = FALSE,
  roman.numeral = FALSE
)
}
\arguments{
\item{dates}{Character vector to be converted to \R{}'s date class.}

\item{day.impute}{Integer between 1 and 31, or NA, or NULL. Day of the month
to be imputed when missing. Defaults to 1. If \code{day.impute = NA}, then
\code{NA} will be imputed for the date and a warning will be raised. If
\code{day.impute = NULL}, the function will fail with an error when day is
missing.}

\item{month.impute}{Integer between 1 and 12, or NA, or NULL. Month to be
imputed when missing. Defaults to 7 (July). If \code{month.impute = NA},
then \code{NA} will be imputed for the entire date and a warning will be
raised.
If \code{month.impute = NULL}, the function will fail with an error when
month is missing.}

\item{format}{Character string specifying date interpretation preference.
Either \code{"dmy"} (day-month-year, default) or \code{"mdy"}
(month-day-year, US format). This setting only affects ambiguous numeric
dates like "01/02/2023". When month names are present or year appears
first, the format is auto-detected regardless of this parameter. Note that
unambiguous dates (e.g., "25/12/2023") are parsed correctly regardless of
the format setting.}

\item{excel}{Logical: Assumes \code{FALSE} by default. If \code{TRUE}, treats
numeric-only dates with more than four digits as Excel serial dates with
1900-01-01 origin, correcting for known Excel date discrepancies.}

\item{roman.numeral}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}} Logical: Defaults
to \code{FALSE}. When \code{TRUE}, attempts to interpret Roman numeral
month indications within datasets. This feature may not handle all cases
correctly.}
}
\value{
A vector of elements belonging to \R{}'s built in \code{Date} class
with the following format \code{yyyy-mm-dd}.
}
\description{
Converts a character vector (or single character object) from inconsistently
formatted dates to \R{}'s \code{Date} class. Supports numerous separators
including /, -, ., or space. Supports numeric, abbreviation, or long-hand
month notation in multiple languages (English, French, German, Spanish,
Portuguese, Russian, Czech, Slovak, Indonesian). Where day of the month has
not been supplied, the first day of the month is imputed by default. Either
DMY or YMD is assumed by default. However, the US system of MDY is supported
via the \code{format} argument.
}
\details{
This function intelligently parses dates by:
\itemize{
\item{Handling mixed separators within the same dataset}
\item{Recognizing month names in multiple languages}
\item{Converting Roman numeral months (experimental)}
\item{Processing Excel serial date numbers}
\item{Automatically detecting YMD format when year appears first}
\item{Smart imputation of missing date components with user control}
}

For comprehensive examples and advanced usage, see \code{browseVignettes("datefixR")}
or the package README at \url{https://docs.ropensci.org/datefixR/}.
}
\examples{
# Basic usage
bad.date <- "02 03 2021"
fix_date_char(bad.date)

# Multiple formats with different separators
mixed_dates <- c(
  "02/05/92", # slash separator, 2-digit year
  "2020-may-01", # hyphen separator, text month
  "1996.05.01", # dot separator
  "02 04 96", # space separator
  "le 3 mars 2013" # French format
)
fix_date_char(mixed_dates)

# Text months in different languages
text_months <- c(
  "15 January 2020", # English
  "15 janvier 2020", # French
  "15 Januar 2020", # German
  "15 enero 2020", # Spanish
  "15 de janeiro de 2020" # Portuguese
)
fix_date_char(text_months)

# Roman numeral months (experimental)
roman_dates <- c("15.VII.2023", "3.XII.1999", "1.I.2000")
fix_date_char(roman_dates, roman.numeral = TRUE)

# Excel serial numbers
excel_serials <- c("44197", "44927") # Excel dates
fix_date_char(excel_serials, excel = TRUE)

# Two-digit years (automatic century detection)
two_digit_years <- c("15/03/99", "15/03/25", "15/03/50")
fix_date_char(two_digit_years) # 1999, 2025, 1950

# MDY format (US style)
us_dates <- c("12/25/2023", "07/04/1776", "02/29/2020")
fix_date_char(us_dates, format = "mdy")

# Incomplete dates with custom imputation
incomplete <- c("2023", "March 2022", "June 2021")
fix_date_char(incomplete, day.impute = 15, month.impute = 6)

}
\seealso{
\code{\link{fix_date_df}} for data frame columns with date data.

For detailed examples and usage patterns, see:
\itemize{
\item{Package vignette: \code{browseVignettes("datefixR")}}
\item{Online documentation: \url{https://docs.ropensci.org/datefixR/articles/datefixR.html}}
\item{Package README: \url{https://docs.ropensci.org/datefixR/}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datefixR-package.R
\docType{package}
\name{datefixR}
\alias{datefixR-package}
\alias{datefixR}
\title{datefixR: Standardize Dates in Different Formats or with Missing Data}
\description{
There are many different formats dates are commonly represented
with: the order of day, month, or year can differ, different separators
("-", "/", or whitespace) can be used, months can be numerical, names, or
abbreviations and year given as two digits or four. \code{datefixR} takes dates
in all these different formats and converts them to \R{}'s built-in date
class. If \code{datefixR} cannot standardize a date, such as because it is too
malformed, then the user is told which date cannot be standardized and the
corresponding ID for the row. \code{datefixR} also allows the imputation of
missing days and months with user-controlled behavior.

Get started by reading \code{vignette("datefixR")}
}
\seealso{
Useful links:
\itemize{
\item \url{https://docs.ropensci.org/datefixR/}
\item \url{https://github.com/ropensci/datefixR/}
\item Report bugs at \url{https://github.com/ropensci/datefixR/issues}
}
}
\author{
\strong{Maintainer}: Nathan Constantine-Cooke \email{nathan.constantine-cooke@ed.ac.uk} (\href{https://orcid.org/0000-0002-4437-8713}{ORCID})

Other contributors:
\itemize{
  \item Jonathan Kitt \email{jonathan.kitt@protonmail.com} [contributor, translator]
  \item Antonio J. Pérez-Luque \email{ajpelu@gmail.com} (\href{https://orcid.org/0000-0002-1747-0469}{ORCID}) [contributor, translator]
  \item Daniel Possenriede \email{possenriede+r@gmail.com} (\href{https://orcid.org/0000-0002-6738-9845}{ORCID}) [contributor, translator]
  \item Michal Lauer \email{michal.lauer.25@gmail.com} [contributor, translator]
  \item Kaique dos S. Alves \email{kaiquedsalves@gmail.com} (\href{https://orcid.org/0000-0001-9187-0252}{ORCID}) [reviewer]
  \item Al-Ahmadgaid B. Asaad \email{alahmadgaid@gmail.com} (\href{https://orcid.org/0000-0003-3784-8593}{ORCID}) [reviewer]
  \item Anatoly Tsyplenkov \email{atsyplenkov@gmail.com} (\href{https://orcid.org/0000-0003-4144-8402}{ORCID}) [contributor, translator]
  \item Chitra M. Saraswati \email{chitra.m.saraswati@gmail.com} (\href{https://orcid.org/0000-0002-8159-0414}{ORCID}) [contributor, translator]
}

}
\keyword{internal}
